"""
Python module generated from Java source file java.util.Collections

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import IOException
from java.io import ObjectInputStream
from java.io import ObjectOutputStream
from java.io import Serializable
from java.lang.reflect import Array
from java.util import *
from java.util.function import BiConsumer
from java.util.function import BiFunction
from java.util.function import Consumer
from java.util.function import Function
from java.util.function import IntFunction
from java.util.function import Predicate
from java.util.function import UnaryOperator
from java.util.stream import IntStream
from java.util.stream import Stream
from java.util.stream import StreamSupport
from jdk.internal.access import SharedSecrets
from typing import Any, Callable, Iterable, Tuple


class Collections:

    EMPTY_SET = EmptySet<>()
    """
    The empty set (immutable).  This set is serializable.

    See
    - .emptySet()
    """
    EMPTY_LIST = EmptyList<>()
    """
    The empty list (immutable).  This list is serializable.

    See
    - .emptyList()
    """
    EMPTY_MAP = EmptyMap<>()
    """
    The empty map (immutable).  This map is serializable.

    See
    - .emptyMap()

    Since
    - 1.3
    """


    @staticmethod
    def sort(list: list["T"]) -> None:
        """
        Sorts the specified list into ascending order, according to the
        Comparable natural ordering of its elements.
        All elements in the list must implement the Comparable
        interface.  Furthermore, all elements in the list must be
        *mutually comparable* (that is, `e1.compareTo(e2)`
        must not throw a `ClassCastException` for any elements
        `e1` and `e2` in the list).
        
        This sort is guaranteed to be *stable*:  equal elements will
        not be reordered as a result of the sort.
        
        The specified list must be modifiable, but need not be resizable.
        
        Type `<T>`: the class of the objects in the list

        Arguments
        - list: the list to be sorted.

        Raises
        - ClassCastException: if the list contains elements that are not
                *mutually comparable* (for example, strings and integers).
        - UnsupportedOperationException: if the specified list's
                list-iterator does not support the `set` operation.
        - IllegalArgumentException: (optional) if the implementation
                detects that the natural ordering of the list elements is
                found to violate the Comparable contract

        See
        - List.sort(Comparator)

        Unknown Tags
        - This implementation defers to the List.sort(Comparator)
        method using the specified list and a `null` comparator.
        """
        ...


    @staticmethod
    def sort(list: list["T"], c: "Comparator"["T"]) -> None:
        """
        Sorts the specified list according to the order induced by the
        specified comparator.  All elements in the list must be *mutually
        comparable* using the specified comparator (that is,
        `c.compare(e1, e2)` must not throw a `ClassCastException`
        for any elements `e1` and `e2` in the list).
        
        This sort is guaranteed to be *stable*:  equal elements will
        not be reordered as a result of the sort.
        
        The specified list must be modifiable, but need not be resizable.
        
        Type `<T>`: the class of the objects in the list

        Arguments
        - list: the list to be sorted.
        - c: the comparator to determine the order of the list.  A
               `null` value indicates that the elements' *natural
               ordering* should be used.

        Raises
        - ClassCastException: if the list contains elements that are not
                *mutually comparable* using the specified comparator.
        - UnsupportedOperationException: if the specified list's
                list-iterator does not support the `set` operation.
        - IllegalArgumentException: (optional) if the comparator is
                found to violate the Comparator contract

        See
        - List.sort(Comparator)

        Unknown Tags
        - This implementation defers to the List.sort(Comparator)
        method using the specified list and comparator.
        """
        ...


    @staticmethod
    def binarySearch(list: list["Comparable"["T"]], key: "T") -> int:
        """
        Searches the specified list for the specified object using the binary
        search algorithm.  The list must be sorted into ascending order
        according to the Comparable natural ordering of its
        elements (as by the .sort(List) method) prior to making this
        call.  If it is not sorted, the results are undefined.  If the list
        contains multiple elements equal to the specified object, there is no
        guarantee which one will be found.
        
        This method runs in log(n) time for a "random access" list (which
        provides near-constant-time positional access).  If the specified list
        does not implement the RandomAccess interface and is large,
        this method will do an iterator-based binary search that performs
        O(n) link traversals and O(log n) element comparisons.
        
        Type `<T>`: the class of the objects in the list

        Arguments
        - list: the list to be searched.
        - key: the key to be searched for.

        Returns
        - the index of the search key, if it is contained in the list;
                otherwise, `(-(*insertion point*) - 1)`.  The
                *insertion point* is defined as the point at which the
                key would be inserted into the list: the index of the first
                element greater than the key, or `list.size()` if all
                elements in the list are less than the specified key.  Note
                that this guarantees that the return value will be &gt;= 0 if
                and only if the key is found.

        Raises
        - ClassCastException: if the list contains elements that are not
                *mutually comparable* (for example, strings and
                integers), or the search key is not mutually comparable
                with the elements of the list.
        """
        ...


    @staticmethod
    def binarySearch(list: list["T"], key: "T", c: "Comparator"["T"]) -> int:
        """
        Searches the specified list for the specified object using the binary
        search algorithm.  The list must be sorted into ascending order
        according to the specified comparator (as by the
        .sort(List, Comparator) sort(List, Comparator)
        method), prior to making this call.  If it is
        not sorted, the results are undefined.  If the list contains multiple
        elements equal to the specified object, there is no guarantee which one
        will be found.
        
        This method runs in log(n) time for a "random access" list (which
        provides near-constant-time positional access).  If the specified list
        does not implement the RandomAccess interface and is large,
        this method will do an iterator-based binary search that performs
        O(n) link traversals and O(log n) element comparisons.
        
        Type `<T>`: the class of the objects in the list

        Arguments
        - list: the list to be searched.
        - key: the key to be searched for.
        - c: the comparator by which the list is ordered.
                A `null` value indicates that the elements'
                Comparable natural ordering should be used.

        Returns
        - the index of the search key, if it is contained in the list;
                otherwise, `(-(*insertion point*) - 1)`.  The
                *insertion point* is defined as the point at which the
                key would be inserted into the list: the index of the first
                element greater than the key, or `list.size()` if all
                elements in the list are less than the specified key.  Note
                that this guarantees that the return value will be &gt;= 0 if
                and only if the key is found.

        Raises
        - ClassCastException: if the list contains elements that are not
                *mutually comparable* using the specified comparator,
                or the search key is not mutually comparable with the
                elements of the list using this comparator.
        """
        ...


    @staticmethod
    def reverse(list: list[Any]) -> None:
        """
        Reverses the order of the elements in the specified list.
        
        This method runs in linear time.

        Arguments
        - list: the list whose elements are to be reversed.

        Raises
        - UnsupportedOperationException: if the specified list or
                its list-iterator does not support the `set` operation.
        """
        ...


    @staticmethod
    def shuffle(list: list[Any]) -> None:
        """
        Randomly permutes the specified list using a default source of
        randomness.  All permutations occur with approximately equal
        likelihood.
        
        The hedge "approximately" is used in the foregoing description because
        default source of randomness is only approximately an unbiased source
        of independently chosen bits. If it were a perfect source of randomly
        chosen bits, then the algorithm would choose permutations with perfect
        uniformity.
        
        This implementation traverses the list backwards, from the last
        element up to the second, repeatedly swapping a randomly selected element
        into the "current position".  Elements are randomly selected from the
        portion of the list that runs from the first element to the current
        position, inclusive.
        
        This method runs in linear time.  If the specified list does not
        implement the RandomAccess interface and is large, this
        implementation dumps the specified list into an array before shuffling
        it, and dumps the shuffled array back into the list.  This avoids the
        quadratic behavior that would result from shuffling a "sequential
        access" list in place.

        Arguments
        - list: the list to be shuffled.

        Raises
        - UnsupportedOperationException: if the specified list or
                its list-iterator does not support the `set` operation.
        """
        ...


    @staticmethod
    def shuffle(list: list[Any], rnd: "Random") -> None:
        """
        Randomly permute the specified list using the specified source of
        randomness.  All permutations occur with equal likelihood
        assuming that the source of randomness is fair.
        
        This implementation traverses the list backwards, from the last element
        up to the second, repeatedly swapping a randomly selected element into
        the "current position".  Elements are randomly selected from the
        portion of the list that runs from the first element to the current
        position, inclusive.
        
        This method runs in linear time.  If the specified list does not
        implement the RandomAccess interface and is large, this
        implementation dumps the specified list into an array before shuffling
        it, and dumps the shuffled array back into the list.  This avoids the
        quadratic behavior that would result from shuffling a "sequential
        access" list in place.

        Arguments
        - list: the list to be shuffled.
        - rnd: the source of randomness to use to shuffle the list.

        Raises
        - UnsupportedOperationException: if the specified list or its
                list-iterator does not support the `set` operation.
        """
        ...


    @staticmethod
    def swap(list: list[Any], i: int, j: int) -> None:
        """
        Swaps the elements at the specified positions in the specified list.
        (If the specified positions are equal, invoking this method leaves
        the list unchanged.)

        Arguments
        - list: The list in which to swap elements.
        - i: the index of one element to be swapped.
        - j: the index of the other element to be swapped.

        Raises
        - IndexOutOfBoundsException: if either `i` or `j`
                is out of range (i &lt; 0 || i &gt;= list.size()
                || j &lt; 0 || j &gt;= list.size()).

        Since
        - 1.4
        """
        ...


    @staticmethod
    def fill(list: list["T"], obj: "T") -> None:
        """
        Replaces all of the elements of the specified list with the specified
        element. 
        
        This method runs in linear time.
        
        Type `<T>`: the class of the objects in the list

        Arguments
        - list: the list to be filled with the specified element.
        - obj: The element with which to fill the specified list.

        Raises
        - UnsupportedOperationException: if the specified list or its
                list-iterator does not support the `set` operation.
        """
        ...


    @staticmethod
    def copy(dest: list["T"], src: list["T"]) -> None:
        """
        Copies all of the elements from one list into another.  After the
        operation, the index of each copied element in the destination list
        will be identical to its index in the source list.  The destination
        list's size must be greater than or equal to the source list's size.
        If it is greater, the remaining elements in the destination list are
        unaffected. 
        
        This method runs in linear time.
        
        Type `<T>`: the class of the objects in the lists

        Arguments
        - dest: The destination list.
        - src: The source list.

        Raises
        - IndexOutOfBoundsException: if the destination list is too small
                to contain the entire source List.
        - UnsupportedOperationException: if the destination list's
                list-iterator does not support the `set` operation.
        """
        ...


    @staticmethod
    def min(coll: Iterable["T"]) -> "T":
        """
        Returns the minimum element of the given collection, according to the
        *natural ordering* of its elements.  All elements in the
        collection must implement the `Comparable` interface.
        Furthermore, all elements in the collection must be *mutually
        comparable* (that is, `e1.compareTo(e2)` must not throw a
        `ClassCastException` for any elements `e1` and
        `e2` in the collection).
        
        This method iterates over the entire collection, hence it requires
        time proportional to the size of the collection.
        
        Type `<T>`: the class of the objects in the collection

        Arguments
        - coll: the collection whose minimum element is to be determined.

        Returns
        - the minimum element of the given collection, according
                to the *natural ordering* of its elements.

        Raises
        - ClassCastException: if the collection contains elements that are
                not *mutually comparable* (for example, strings and
                integers).
        - NoSuchElementException: if the collection is empty.

        See
        - Comparable
        """
        ...


    @staticmethod
    def min(coll: Iterable["T"], comp: "Comparator"["T"]) -> "T":
        """
        Returns the minimum element of the given collection, according to the
        order induced by the specified comparator.  All elements in the
        collection must be *mutually comparable* by the specified
        comparator (that is, `comp.compare(e1, e2)` must not throw a
        `ClassCastException` for any elements `e1` and
        `e2` in the collection).
        
        This method iterates over the entire collection, hence it requires
        time proportional to the size of the collection.
        
        Type `<T>`: the class of the objects in the collection

        Arguments
        - coll: the collection whose minimum element is to be determined.
        - comp: the comparator with which to determine the minimum element.
                A `null` value indicates that the elements' *natural
                ordering* should be used.

        Returns
        - the minimum element of the given collection, according
                to the specified comparator.

        Raises
        - ClassCastException: if the collection contains elements that are
                not *mutually comparable* using the specified comparator.
        - NoSuchElementException: if the collection is empty.

        See
        - Comparable
        """
        ...


    @staticmethod
    def max(coll: Iterable["T"]) -> "T":
        """
        Returns the maximum element of the given collection, according to the
        *natural ordering* of its elements.  All elements in the
        collection must implement the `Comparable` interface.
        Furthermore, all elements in the collection must be *mutually
        comparable* (that is, `e1.compareTo(e2)` must not throw a
        `ClassCastException` for any elements `e1` and
        `e2` in the collection).
        
        This method iterates over the entire collection, hence it requires
        time proportional to the size of the collection.
        
        Type `<T>`: the class of the objects in the collection

        Arguments
        - coll: the collection whose maximum element is to be determined.

        Returns
        - the maximum element of the given collection, according
                to the *natural ordering* of its elements.

        Raises
        - ClassCastException: if the collection contains elements that are
                not *mutually comparable* (for example, strings and
                integers).
        - NoSuchElementException: if the collection is empty.

        See
        - Comparable
        """
        ...


    @staticmethod
    def max(coll: Iterable["T"], comp: "Comparator"["T"]) -> "T":
        """
        Returns the maximum element of the given collection, according to the
        order induced by the specified comparator.  All elements in the
        collection must be *mutually comparable* by the specified
        comparator (that is, `comp.compare(e1, e2)` must not throw a
        `ClassCastException` for any elements `e1` and
        `e2` in the collection).
        
        This method iterates over the entire collection, hence it requires
        time proportional to the size of the collection.
        
        Type `<T>`: the class of the objects in the collection

        Arguments
        - coll: the collection whose maximum element is to be determined.
        - comp: the comparator with which to determine the maximum element.
                A `null` value indicates that the elements' *natural
               ordering* should be used.

        Returns
        - the maximum element of the given collection, according
                to the specified comparator.

        Raises
        - ClassCastException: if the collection contains elements that are
                not *mutually comparable* using the specified comparator.
        - NoSuchElementException: if the collection is empty.

        See
        - Comparable
        """
        ...


    @staticmethod
    def rotate(list: list[Any], distance: int) -> None:
        """
        Rotates the elements in the specified list by the specified distance.
        After calling this method, the element at index `i` will be
        the element previously at index `(i - distance)` mod
        `list.size()`, for all values of `i` between `0`
        and `list.size()-1`, inclusive.  (This method has no effect on
        the size of the list.)
        
        For example, suppose `list` comprises`[t, a, n, k, s]`.
        After invoking `Collections.rotate(list, 1)` (or
        `Collections.rotate(list, -4)`), `list` will comprise
        `[s, t, a, n, k]`.
        
        Note that this method can usefully be applied to sublists to
        move one or more elements within a list while preserving the
        order of the remaining elements.  For example, the following idiom
        moves the element at index `j` forward to position
        `k` (which must be greater than or equal to `j`):
        ```
            Collections.rotate(list.subList(j, k+1), -1);
        ```
        To make this concrete, suppose `list` comprises
        `[a, b, c, d, e]`.  To move the element at index `1`
        (`b`) forward two positions, perform the following invocation:
        ```
            Collections.rotate(l.subList(1, 4), -1);
        ```
        The resulting list is `[a, c, d, b, e]`.
        
        To move more than one element forward, increase the absolute value
        of the rotation distance.  To move elements backward, use a positive
        shift distance.
        
        If the specified list is small or implements the RandomAccess interface, this implementation exchanges the first
        element into the location it should go, and then repeatedly exchanges
        the displaced element into the location it should go until a displaced
        element is swapped into the first element.  If necessary, the process
        is repeated on the second and successive elements, until the rotation
        is complete.  If the specified list is large and doesn't implement the
        `RandomAccess` interface, this implementation breaks the
        list into two sublist views around index `-distance mod size`.
        Then the .reverse(List) method is invoked on each sublist view,
        and finally it is invoked on the entire list.  For a more complete
        description of both algorithms, see Section 2.3 of Jon Bentley's
        *Programming Pearls* (Addison-Wesley, 1986).

        Arguments
        - list: the list to be rotated.
        - distance: the distance to rotate the list.  There are no
               constraints on this value; it may be zero, negative, or
               greater than `list.size()`.

        Raises
        - UnsupportedOperationException: if the specified list or
                its list-iterator does not support the `set` operation.

        Since
        - 1.4
        """
        ...


    @staticmethod
    def replaceAll(list: list["T"], oldVal: "T", newVal: "T") -> bool:
        """
        Replaces all occurrences of one specified value in a list with another.
        More formally, replaces with `newVal` each element `e`
        in `list` such that
        `(oldVal==null ? e==null : oldVal.equals(e))`.
        (This method has no effect on the size of the list.)
        
        Type `<T>`: the class of the objects in the list

        Arguments
        - list: the list in which replacement is to occur.
        - oldVal: the old value to be replaced.
        - newVal: the new value with which `oldVal` is to be
               replaced.

        Returns
        - `True` if `list` contained one or more elements
                `e` such that
                `(oldVal==null ?  e==null : oldVal.equals(e))`.

        Raises
        - UnsupportedOperationException: if the specified list or
                its list-iterator does not support the `set` operation.

        Since
        - 1.4
        """
        ...


    @staticmethod
    def indexOfSubList(source: list[Any], target: list[Any]) -> int:
        """
        Returns the starting position of the first occurrence of the specified
        target list within the specified source list, or -1 if there is no
        such occurrence.  More formally, returns the lowest index `i`
        such that `source.subList(i, i+target.size()).equals(target)`,
        or -1 if there is no such index.  (Returns -1 if
        `target.size() > source.size()`)
        
        This implementation uses the "brute force" technique of scanning
        over the source list, looking for a match with the target at each
        location in turn.

        Arguments
        - source: the list in which to search for the first occurrence
               of `target`.
        - target: the list to search for as a subList of `source`.

        Returns
        - the starting position of the first occurrence of the specified
                target list within the specified source list, or -1 if there
                is no such occurrence.

        Since
        - 1.4
        """
        ...


    @staticmethod
    def lastIndexOfSubList(source: list[Any], target: list[Any]) -> int:
        """
        Returns the starting position of the last occurrence of the specified
        target list within the specified source list, or -1 if there is no such
        occurrence.  More formally, returns the highest index `i`
        such that `source.subList(i, i+target.size()).equals(target)`,
        or -1 if there is no such index.  (Returns -1 if
        `target.size() > source.size()`)
        
        This implementation uses the "brute force" technique of iterating
        over the source list, looking for a match with the target at each
        location in turn.

        Arguments
        - source: the list in which to search for the last occurrence
               of `target`.
        - target: the list to search for as a subList of `source`.

        Returns
        - the starting position of the last occurrence of the specified
                target list within the specified source list, or -1 if there
                is no such occurrence.

        Since
        - 1.4
        """
        ...


    @staticmethod
    def unmodifiableCollection(c: Iterable["T"]) -> Iterable["T"]:
        """
        Returns an <a href="Collection.html#unmodview">unmodifiable view</a> of the
        specified collection. Query operations on the returned collection "read through"
        to the specified collection, and attempts to modify the returned
        collection, whether direct or via its iterator, result in an
        `UnsupportedOperationException`.
        
        The returned collection does *not* pass the hashCode and equals
        operations through to the backing collection, but relies on
        `Object`'s `equals` and `hashCode` methods.  This
        is necessary to preserve the contracts of these operations in the case
        that the backing collection is a set or a list.
        
        The returned collection will be serializable if the specified collection
        is serializable.
        
        Type `<T>`: the class of the objects in the collection

        Arguments
        - c: the collection for which an unmodifiable view is to be
                returned.

        Returns
        - an unmodifiable view of the specified collection.

        Unknown Tags
        - This method may return its argument if the argument is already unmodifiable.
        """
        ...


    @staticmethod
    def unmodifiableSet(s: set["T"]) -> set["T"]:
        """
        Returns an <a href="Collection.html#unmodview">unmodifiable view</a> of the
        specified set. Query operations on the returned set "read through" to the specified
        set, and attempts to modify the returned set, whether direct or via its
        iterator, result in an `UnsupportedOperationException`.
        
        The returned set will be serializable if the specified set
        is serializable.
        
        Type `<T>`: the class of the objects in the set

        Arguments
        - s: the set for which an unmodifiable view is to be returned.

        Returns
        - an unmodifiable view of the specified set.

        Unknown Tags
        - This method may return its argument if the argument is already unmodifiable.
        """
        ...


    @staticmethod
    def unmodifiableSortedSet(s: "SortedSet"["T"]) -> "SortedSet"["T"]:
        """
        Returns an <a href="Collection.html#unmodview">unmodifiable view</a> of the
        specified sorted set. Query operations on the returned sorted set "read
        through" to the specified sorted set.  Attempts to modify the returned
        sorted set, whether direct, via its iterator, or via its
        `subSet`, `headSet`, or `tailSet` views, result in
        an `UnsupportedOperationException`.
        
        The returned sorted set will be serializable if the specified sorted set
        is serializable.
        
        Type `<T>`: the class of the objects in the set

        Arguments
        - s: the sorted set for which an unmodifiable view is to be
               returned.

        Returns
        - an unmodifiable view of the specified sorted set.

        Unknown Tags
        - This method may return its argument if the argument is already unmodifiable.
        """
        ...


    @staticmethod
    def unmodifiableNavigableSet(s: "NavigableSet"["T"]) -> "NavigableSet"["T"]:
        """
        Returns an <a href="Collection.html#unmodview">unmodifiable view</a> of the
        specified navigable set. Query operations on the returned navigable set "read
        through" to the specified navigable set.  Attempts to modify the returned
        navigable set, whether direct, via its iterator, or via its
        `subSet`, `headSet`, or `tailSet` views, result in
        an `UnsupportedOperationException`.
        
        The returned navigable set will be serializable if the specified
        navigable set is serializable.
        
        Type `<T>`: the class of the objects in the set

        Arguments
        - s: the navigable set for which an unmodifiable view is to be
               returned

        Returns
        - an unmodifiable view of the specified navigable set

        Since
        - 1.8

        Unknown Tags
        - This method may return its argument if the argument is already unmodifiable.
        """
        ...


    @staticmethod
    def unmodifiableList(list: list["T"]) -> list["T"]:
        """
        Returns an <a href="Collection.html#unmodview">unmodifiable view</a> of the
        specified list. Query operations on the returned list "read through" to the
        specified list, and attempts to modify the returned list, whether
        direct or via its iterator, result in an
        `UnsupportedOperationException`.
        
        The returned list will be serializable if the specified list
        is serializable. Similarly, the returned list will implement
        RandomAccess if the specified list does.
        
        Type `<T>`: the class of the objects in the list

        Arguments
        - list: the list for which an unmodifiable view is to be returned.

        Returns
        - an unmodifiable view of the specified list.

        Unknown Tags
        - This method may return its argument if the argument is already unmodifiable.
        """
        ...


    @staticmethod
    def unmodifiableMap(m: dict["K", "V"]) -> dict["K", "V"]:
        """
        Returns an <a href="Collection.html#unmodview">unmodifiable view</a> of the
        specified map. Query operations on the returned map "read through"
        to the specified map, and attempts to modify the returned
        map, whether direct or via its collection views, result in an
        `UnsupportedOperationException`.
        
        The returned map will be serializable if the specified map
        is serializable.
        
        Type `<K>`: the class of the map keys
        
        Type `<V>`: the class of the map values

        Arguments
        - m: the map for which an unmodifiable view is to be returned.

        Returns
        - an unmodifiable view of the specified map.

        Unknown Tags
        - This method may return its argument if the argument is already unmodifiable.
        """
        ...


    @staticmethod
    def unmodifiableSortedMap(m: "SortedMap"["K", "V"]) -> "SortedMap"["K", "V"]:
        """
        Returns an <a href="Collection.html#unmodview">unmodifiable view</a> of the
        specified sorted map. Query operations on the returned sorted map "read through"
        to the specified sorted map.  Attempts to modify the returned
        sorted map, whether direct, via its collection views, or via its
        `subMap`, `headMap`, or `tailMap` views, result in
        an `UnsupportedOperationException`.
        
        The returned sorted map will be serializable if the specified sorted map
        is serializable.
        
        Type `<K>`: the class of the map keys
        
        Type `<V>`: the class of the map values

        Arguments
        - m: the sorted map for which an unmodifiable view is to be
               returned.

        Returns
        - an unmodifiable view of the specified sorted map.

        Unknown Tags
        - This method may return its argument if the argument is already unmodifiable.
        """
        ...


    @staticmethod
    def unmodifiableNavigableMap(m: "NavigableMap"["K", "V"]) -> "NavigableMap"["K", "V"]:
        """
        Returns an <a href="Collection.html#unmodview">unmodifiable view</a> of the
        specified navigable map. Query operations on the returned navigable map "read
        through" to the specified navigable map.  Attempts to modify the returned
        navigable map, whether direct, via its collection views, or via its
        `subMap`, `headMap`, or `tailMap` views, result in
        an `UnsupportedOperationException`.
        
        The returned navigable map will be serializable if the specified
        navigable map is serializable.
        
        Type `<K>`: the class of the map keys
        
        Type `<V>`: the class of the map values

        Arguments
        - m: the navigable map for which an unmodifiable view is to be
               returned

        Returns
        - an unmodifiable view of the specified navigable map

        Since
        - 1.8

        Unknown Tags
        - This method may return its argument if the argument is already unmodifiable.
        """
        ...


    @staticmethod
    def synchronizedCollection(c: Iterable["T"]) -> Iterable["T"]:
        """
        Returns a synchronized (thread-safe) collection backed by the specified
        collection.  In order to guarantee serial access, it is critical that
        <strong>all</strong> access to the backing collection is accomplished
        through the returned collection.
        
        It is imperative that the user manually synchronize on the returned
        collection when traversing it via Iterator, Spliterator
        or Stream:
        ```
         Collection c = Collections.synchronizedCollection(myCollection);
            ...
         synchronized (c) {
             Iterator i = c.iterator(); // Must be in the synchronized block
             while (i.hasNext())
                foo(i.next());
         }
        ```
        Failure to follow this advice may result in non-deterministic behavior.
        
        The returned collection does *not* pass the `hashCode`
        and `equals` operations through to the backing collection, but
        relies on `Object`'s equals and hashCode methods.  This is
        necessary to preserve the contracts of these operations in the case
        that the backing collection is a set or a list.
        
        The returned collection will be serializable if the specified collection
        is serializable.
        
        Type `<T>`: the class of the objects in the collection

        Arguments
        - c: the collection to be "wrapped" in a synchronized collection.

        Returns
        - a synchronized view of the specified collection.
        """
        ...


    @staticmethod
    def synchronizedSet(s: set["T"]) -> set["T"]:
        """
        Returns a synchronized (thread-safe) set backed by the specified
        set.  In order to guarantee serial access, it is critical that
        <strong>all</strong> access to the backing set is accomplished
        through the returned set.
        
        It is imperative that the user manually synchronize on the returned
        collection when traversing it via Iterator, Spliterator
        or Stream:
        ```
         Set s = Collections.synchronizedSet(new HashSet());
             ...
         synchronized (s) {
             Iterator i = s.iterator(); // Must be in the synchronized block
             while (i.hasNext())
                 foo(i.next());
         }
        ```
        Failure to follow this advice may result in non-deterministic behavior.
        
        The returned set will be serializable if the specified set is
        serializable.
        
        Type `<T>`: the class of the objects in the set

        Arguments
        - s: the set to be "wrapped" in a synchronized set.

        Returns
        - a synchronized view of the specified set.
        """
        ...


    @staticmethod
    def synchronizedSortedSet(s: "SortedSet"["T"]) -> "SortedSet"["T"]:
        """
        Returns a synchronized (thread-safe) sorted set backed by the specified
        sorted set.  In order to guarantee serial access, it is critical that
        <strong>all</strong> access to the backing sorted set is accomplished
        through the returned sorted set (or its views).
        
        It is imperative that the user manually synchronize on the returned
        sorted set when traversing it or any of its `subSet`,
        `headSet`, or `tailSet` views via Iterator,
        Spliterator or Stream:
        ```
         SortedSet s = Collections.synchronizedSortedSet(new TreeSet());
             ...
         synchronized (s) {
             Iterator i = s.iterator(); // Must be in the synchronized block
             while (i.hasNext())
                 foo(i.next());
         }
        ```
        or:
        ```
         SortedSet s = Collections.synchronizedSortedSet(new TreeSet());
         SortedSet s2 = s.headSet(foo);
             ...
         synchronized (s) {  // Note: s, not s2!!!
             Iterator i = s2.iterator(); // Must be in the synchronized block
             while (i.hasNext())
                 foo(i.next());
         }
        ```
        Failure to follow this advice may result in non-deterministic behavior.
        
        The returned sorted set will be serializable if the specified
        sorted set is serializable.
        
        Type `<T>`: the class of the objects in the set

        Arguments
        - s: the sorted set to be "wrapped" in a synchronized sorted set.

        Returns
        - a synchronized view of the specified sorted set.
        """
        ...


    @staticmethod
    def synchronizedNavigableSet(s: "NavigableSet"["T"]) -> "NavigableSet"["T"]:
        """
        Returns a synchronized (thread-safe) navigable set backed by the
        specified navigable set.  In order to guarantee serial access, it is
        critical that <strong>all</strong> access to the backing navigable set is
        accomplished through the returned navigable set (or its views).
        
        It is imperative that the user manually synchronize on the returned
        navigable set when traversing it, or any of its `subSet`,
        `headSet`, or `tailSet` views, via Iterator,
        Spliterator or Stream:
        ```
         NavigableSet s = Collections.synchronizedNavigableSet(new TreeSet());
             ...
         synchronized (s) {
             Iterator i = s.iterator(); // Must be in the synchronized block
             while (i.hasNext())
                 foo(i.next());
         }
        ```
        or:
        ```
         NavigableSet s = Collections.synchronizedNavigableSet(new TreeSet());
         NavigableSet s2 = s.headSet(foo, True);
             ...
         synchronized (s) {  // Note: s, not s2!!!
             Iterator i = s2.iterator(); // Must be in the synchronized block
             while (i.hasNext())
                 foo(i.next());
         }
        ```
        Failure to follow this advice may result in non-deterministic behavior.
        
        The returned navigable set will be serializable if the specified
        navigable set is serializable.
        
        Type `<T>`: the class of the objects in the set

        Arguments
        - s: the navigable set to be "wrapped" in a synchronized navigable
        set

        Returns
        - a synchronized view of the specified navigable set

        Since
        - 1.8
        """
        ...


    @staticmethod
    def synchronizedList(list: list["T"]) -> list["T"]:
        """
        Returns a synchronized (thread-safe) list backed by the specified
        list.  In order to guarantee serial access, it is critical that
        <strong>all</strong> access to the backing list is accomplished
        through the returned list.
        
        It is imperative that the user manually synchronize on the returned
        list when traversing it via Iterator, Spliterator
        or Stream:
        ```
         List list = Collections.synchronizedList(new ArrayList());
             ...
         synchronized (list) {
             Iterator i = list.iterator(); // Must be in synchronized block
             while (i.hasNext())
                 foo(i.next());
         }
        ```
        Failure to follow this advice may result in non-deterministic behavior.
        
        The returned list will be serializable if the specified list is
        serializable.
        
        Type `<T>`: the class of the objects in the list

        Arguments
        - list: the list to be "wrapped" in a synchronized list.

        Returns
        - a synchronized view of the specified list.
        """
        ...


    @staticmethod
    def synchronizedMap(m: dict["K", "V"]) -> dict["K", "V"]:
        """
        Returns a synchronized (thread-safe) map backed by the specified
        map.  In order to guarantee serial access, it is critical that
        <strong>all</strong> access to the backing map is accomplished
        through the returned map.
        
        It is imperative that the user manually synchronize on the returned
        map when traversing any of its collection views via Iterator,
        Spliterator or Stream:
        ```
         Map m = Collections.synchronizedMap(new HashMap());
             ...
         Set s = m.keySet();  // Needn't be in synchronized block
             ...
         synchronized (m) {  // Synchronizing on m, not s!
             Iterator i = s.iterator(); // Must be in synchronized block
             while (i.hasNext())
                 foo(i.next());
         }
        ```
        Failure to follow this advice may result in non-deterministic behavior.
        
        The returned map will be serializable if the specified map is
        serializable.
        
        Type `<K>`: the class of the map keys
        
        Type `<V>`: the class of the map values

        Arguments
        - m: the map to be "wrapped" in a synchronized map.

        Returns
        - a synchronized view of the specified map.
        """
        ...


    @staticmethod
    def synchronizedSortedMap(m: "SortedMap"["K", "V"]) -> "SortedMap"["K", "V"]:
        """
        Returns a synchronized (thread-safe) sorted map backed by the specified
        sorted map.  In order to guarantee serial access, it is critical that
        <strong>all</strong> access to the backing sorted map is accomplished
        through the returned sorted map (or its views).
        
        It is imperative that the user manually synchronize on the returned
        sorted map when traversing any of its collection views, or the
        collections views of any of its `subMap`, `headMap` or
        `tailMap` views, via Iterator, Spliterator or
        Stream:
        ```
         SortedMap m = Collections.synchronizedSortedMap(new TreeMap());
             ...
         Set s = m.keySet();  // Needn't be in synchronized block
             ...
         synchronized (m) {  // Synchronizing on m, not s!
             Iterator i = s.iterator(); // Must be in synchronized block
             while (i.hasNext())
                 foo(i.next());
         }
        ```
        or:
        ```
         SortedMap m = Collections.synchronizedSortedMap(new TreeMap());
         SortedMap m2 = m.subMap(foo, bar);
             ...
         Set s2 = m2.keySet();  // Needn't be in synchronized block
             ...
         synchronized (m) {  // Synchronizing on m, not m2 or s2!
             Iterator i = s2.iterator(); // Must be in synchronized block
             while (i.hasNext())
                 foo(i.next());
         }
        ```
        Failure to follow this advice may result in non-deterministic behavior.
        
        The returned sorted map will be serializable if the specified
        sorted map is serializable.
        
        Type `<K>`: the class of the map keys
        
        Type `<V>`: the class of the map values

        Arguments
        - m: the sorted map to be "wrapped" in a synchronized sorted map.

        Returns
        - a synchronized view of the specified sorted map.
        """
        ...


    @staticmethod
    def synchronizedNavigableMap(m: "NavigableMap"["K", "V"]) -> "NavigableMap"["K", "V"]:
        """
        Returns a synchronized (thread-safe) navigable map backed by the
        specified navigable map.  In order to guarantee serial access, it is
        critical that <strong>all</strong> access to the backing navigable map is
        accomplished through the returned navigable map (or its views).
        
        It is imperative that the user manually synchronize on the returned
        navigable map when traversing any of its collection views, or the
        collections views of any of its `subMap`, `headMap` or
        `tailMap` views, via Iterator, Spliterator or
        Stream:
        ```
         NavigableMap m = Collections.synchronizedNavigableMap(new TreeMap());
             ...
         Set s = m.keySet();  // Needn't be in synchronized block
             ...
         synchronized (m) {  // Synchronizing on m, not s!
             Iterator i = s.iterator(); // Must be in synchronized block
             while (i.hasNext())
                 foo(i.next());
         }
        ```
        or:
        ```
         NavigableMap m = Collections.synchronizedNavigableMap(new TreeMap());
         NavigableMap m2 = m.subMap(foo, True, bar, False);
             ...
         Set s2 = m2.keySet();  // Needn't be in synchronized block
             ...
         synchronized (m) {  // Synchronizing on m, not m2 or s2!
             Iterator i = s.iterator(); // Must be in synchronized block
             while (i.hasNext())
                 foo(i.next());
         }
        ```
        Failure to follow this advice may result in non-deterministic behavior.
        
        The returned navigable map will be serializable if the specified
        navigable map is serializable.
        
        Type `<K>`: the class of the map keys
        
        Type `<V>`: the class of the map values

        Arguments
        - m: the navigable map to be "wrapped" in a synchronized navigable
                     map

        Returns
        - a synchronized view of the specified navigable map.

        Since
        - 1.8
        """
        ...


    @staticmethod
    def checkedCollection(c: Iterable["E"], type: type["E"]) -> Iterable["E"]:
        """
        Returns a dynamically typesafe view of the specified collection.
        Any attempt to insert an element of the wrong type will result in an
        immediate ClassCastException.  Assuming a collection
        contains no incorrectly typed elements prior to the time a
        dynamically typesafe view is generated, and that all subsequent
        access to the collection takes place through the view, it is
        *guaranteed* that the collection cannot contain an incorrectly
        typed element.
        
        The generics mechanism in the language provides compile-time
        (static) type checking, but it is possible to defeat this mechanism
        with unchecked casts.  Usually this is not a problem, as the compiler
        issues warnings on all such unchecked operations.  There are, however,
        times when static type checking alone is not sufficient.  For example,
        suppose a collection is passed to a third-party library and it is
        imperative that the library code not corrupt the collection by
        inserting an element of the wrong type.
        
        Another use of dynamically typesafe views is debugging.  Suppose a
        program fails with a `ClassCastException`, indicating that an
        incorrectly typed element was put into a parameterized collection.
        Unfortunately, the exception can occur at any time after the erroneous
        element is inserted, so it typically provides little or no information
        as to the real source of the problem.  If the problem is reproducible,
        one can quickly determine its source by temporarily modifying the
        program to wrap the collection with a dynamically typesafe view.
        For example, this declaration:
         ``` `Collection<String> c = new HashSet<>();````
        may be replaced temporarily by this one:
         ``` `Collection<String> c = Collections.checkedCollection(
                new HashSet<>(), String.class);````
        Running the program again will cause it to fail at the point where
        an incorrectly typed element is inserted into the collection, clearly
        identifying the source of the problem.  Once the problem is fixed, the
        modified declaration may be reverted back to the original.
        
        The returned collection does *not* pass the hashCode and equals
        operations through to the backing collection, but relies on
        `Object`'s `equals` and `hashCode` methods.  This
        is necessary to preserve the contracts of these operations in the case
        that the backing collection is a set or a list.
        
        The returned collection will be serializable if the specified
        collection is serializable.
        
        Since `null` is considered to be a value of any reference
        type, the returned collection permits insertion of null elements
        whenever the backing collection does.
        
        Type `<E>`: the class of the objects in the collection

        Arguments
        - c: the collection for which a dynamically typesafe view is to be
                 returned
        - type: the type of element that `c` is permitted to hold

        Returns
        - a dynamically typesafe view of the specified collection

        Since
        - 1.5
        """
        ...


    @staticmethod
    def checkedQueue(queue: "Queue"["E"], type: type["E"]) -> "Queue"["E"]:
        """
        Returns a dynamically typesafe view of the specified queue.
        Any attempt to insert an element of the wrong type will result in
        an immediate ClassCastException.  Assuming a queue contains
        no incorrectly typed elements prior to the time a dynamically typesafe
        view is generated, and that all subsequent access to the queue
        takes place through the view, it is *guaranteed* that the
        queue cannot contain an incorrectly typed element.
        
        A discussion of the use of dynamically typesafe views may be
        found in the documentation for the .checkedCollection
        checkedCollection method.
        
        The returned queue will be serializable if the specified queue
        is serializable.
        
        Since `null` is considered to be a value of any reference
        type, the returned queue permits insertion of `null` elements
        whenever the backing queue does.
        
        Type `<E>`: the class of the objects in the queue

        Arguments
        - queue: the queue for which a dynamically typesafe view is to be
                    returned
        - type: the type of element that `queue` is permitted to hold

        Returns
        - a dynamically typesafe view of the specified queue

        Since
        - 1.8
        """
        ...


    @staticmethod
    def checkedSet(s: set["E"], type: type["E"]) -> set["E"]:
        """
        Returns a dynamically typesafe view of the specified set.
        Any attempt to insert an element of the wrong type will result in
        an immediate ClassCastException.  Assuming a set contains
        no incorrectly typed elements prior to the time a dynamically typesafe
        view is generated, and that all subsequent access to the set
        takes place through the view, it is *guaranteed* that the
        set cannot contain an incorrectly typed element.
        
        A discussion of the use of dynamically typesafe views may be
        found in the documentation for the .checkedCollection
        checkedCollection method.
        
        The returned set will be serializable if the specified set is
        serializable.
        
        Since `null` is considered to be a value of any reference
        type, the returned set permits insertion of null elements whenever
        the backing set does.
        
        Type `<E>`: the class of the objects in the set

        Arguments
        - s: the set for which a dynamically typesafe view is to be
                 returned
        - type: the type of element that `s` is permitted to hold

        Returns
        - a dynamically typesafe view of the specified set

        Since
        - 1.5
        """
        ...


    @staticmethod
    def checkedSortedSet(s: "SortedSet"["E"], type: type["E"]) -> "SortedSet"["E"]:
        """
        Returns a dynamically typesafe view of the specified sorted set.
        Any attempt to insert an element of the wrong type will result in an
        immediate ClassCastException.  Assuming a sorted set
        contains no incorrectly typed elements prior to the time a
        dynamically typesafe view is generated, and that all subsequent
        access to the sorted set takes place through the view, it is
        *guaranteed* that the sorted set cannot contain an incorrectly
        typed element.
        
        A discussion of the use of dynamically typesafe views may be
        found in the documentation for the .checkedCollection
        checkedCollection method.
        
        The returned sorted set will be serializable if the specified sorted
        set is serializable.
        
        Since `null` is considered to be a value of any reference
        type, the returned sorted set permits insertion of null elements
        whenever the backing sorted set does.
        
        Type `<E>`: the class of the objects in the set

        Arguments
        - s: the sorted set for which a dynamically typesafe view is to be
                 returned
        - type: the type of element that `s` is permitted to hold

        Returns
        - a dynamically typesafe view of the specified sorted set

        Since
        - 1.5
        """
        ...


    @staticmethod
    def checkedNavigableSet(s: "NavigableSet"["E"], type: type["E"]) -> "NavigableSet"["E"]:
        """
        Returns a dynamically typesafe view of the specified navigable set.
        Any attempt to insert an element of the wrong type will result in an
        immediate ClassCastException.  Assuming a navigable set
        contains no incorrectly typed elements prior to the time a
        dynamically typesafe view is generated, and that all subsequent
        access to the navigable set takes place through the view, it is
        *guaranteed* that the navigable set cannot contain an incorrectly
        typed element.
        
        A discussion of the use of dynamically typesafe views may be
        found in the documentation for the .checkedCollection
        checkedCollection method.
        
        The returned navigable set will be serializable if the specified
        navigable set is serializable.
        
        Since `null` is considered to be a value of any reference
        type, the returned navigable set permits insertion of null elements
        whenever the backing sorted set does.
        
        Type `<E>`: the class of the objects in the set

        Arguments
        - s: the navigable set for which a dynamically typesafe view is to be
                 returned
        - type: the type of element that `s` is permitted to hold

        Returns
        - a dynamically typesafe view of the specified navigable set

        Since
        - 1.8
        """
        ...


    @staticmethod
    def checkedList(list: list["E"], type: type["E"]) -> list["E"]:
        """
        Returns a dynamically typesafe view of the specified list.
        Any attempt to insert an element of the wrong type will result in
        an immediate ClassCastException.  Assuming a list contains
        no incorrectly typed elements prior to the time a dynamically typesafe
        view is generated, and that all subsequent access to the list
        takes place through the view, it is *guaranteed* that the
        list cannot contain an incorrectly typed element.
        
        A discussion of the use of dynamically typesafe views may be
        found in the documentation for the .checkedCollection
        checkedCollection method.
        
        The returned list will be serializable if the specified list
        is serializable.
        
        Since `null` is considered to be a value of any reference
        type, the returned list permits insertion of null elements whenever
        the backing list does.
        
        Type `<E>`: the class of the objects in the list

        Arguments
        - list: the list for which a dynamically typesafe view is to be
                    returned
        - type: the type of element that `list` is permitted to hold

        Returns
        - a dynamically typesafe view of the specified list

        Since
        - 1.5
        """
        ...


    @staticmethod
    def checkedMap(m: dict["K", "V"], keyType: type["K"], valueType: type["V"]) -> dict["K", "V"]:
        """
        Returns a dynamically typesafe view of the specified map.
        Any attempt to insert a mapping whose key or value have the wrong
        type will result in an immediate ClassCastException.
        Similarly, any attempt to modify the value currently associated with
        a key will result in an immediate ClassCastException,
        whether the modification is attempted directly through the map
        itself, or through a Map.Entry instance obtained from the
        map's Map.entrySet() entry set view.
        
        Assuming a map contains no incorrectly typed keys or values
        prior to the time a dynamically typesafe view is generated, and
        that all subsequent access to the map takes place through the view
        (or one of its collection views), it is *guaranteed* that the
        map cannot contain an incorrectly typed key or value.
        
        A discussion of the use of dynamically typesafe views may be
        found in the documentation for the .checkedCollection
        checkedCollection method.
        
        The returned map will be serializable if the specified map is
        serializable.
        
        Since `null` is considered to be a value of any reference
        type, the returned map permits insertion of null keys or values
        whenever the backing map does.
        
        Type `<K>`: the class of the map keys
        
        Type `<V>`: the class of the map values

        Arguments
        - m: the map for which a dynamically typesafe view is to be
                 returned
        - keyType: the type of key that `m` is permitted to hold
        - valueType: the type of value that `m` is permitted to hold

        Returns
        - a dynamically typesafe view of the specified map

        Since
        - 1.5
        """
        ...


    @staticmethod
    def checkedSortedMap(m: "SortedMap"["K", "V"], keyType: type["K"], valueType: type["V"]) -> "SortedMap"["K", "V"]:
        """
        Returns a dynamically typesafe view of the specified sorted map.
        Any attempt to insert a mapping whose key or value have the wrong
        type will result in an immediate ClassCastException.
        Similarly, any attempt to modify the value currently associated with
        a key will result in an immediate ClassCastException,
        whether the modification is attempted directly through the map
        itself, or through a Map.Entry instance obtained from the
        map's Map.entrySet() entry set view.
        
        Assuming a map contains no incorrectly typed keys or values
        prior to the time a dynamically typesafe view is generated, and
        that all subsequent access to the map takes place through the view
        (or one of its collection views), it is *guaranteed* that the
        map cannot contain an incorrectly typed key or value.
        
        A discussion of the use of dynamically typesafe views may be
        found in the documentation for the .checkedCollection
        checkedCollection method.
        
        The returned map will be serializable if the specified map is
        serializable.
        
        Since `null` is considered to be a value of any reference
        type, the returned map permits insertion of null keys or values
        whenever the backing map does.
        
        Type `<K>`: the class of the map keys
        
        Type `<V>`: the class of the map values

        Arguments
        - m: the map for which a dynamically typesafe view is to be
                 returned
        - keyType: the type of key that `m` is permitted to hold
        - valueType: the type of value that `m` is permitted to hold

        Returns
        - a dynamically typesafe view of the specified map

        Since
        - 1.5
        """
        ...


    @staticmethod
    def checkedNavigableMap(m: "NavigableMap"["K", "V"], keyType: type["K"], valueType: type["V"]) -> "NavigableMap"["K", "V"]:
        """
        Returns a dynamically typesafe view of the specified navigable map.
        Any attempt to insert a mapping whose key or value have the wrong
        type will result in an immediate ClassCastException.
        Similarly, any attempt to modify the value currently associated with
        a key will result in an immediate ClassCastException,
        whether the modification is attempted directly through the map
        itself, or through a Map.Entry instance obtained from the
        map's Map.entrySet() entry set view.
        
        Assuming a map contains no incorrectly typed keys or values
        prior to the time a dynamically typesafe view is generated, and
        that all subsequent access to the map takes place through the view
        (or one of its collection views), it is *guaranteed* that the
        map cannot contain an incorrectly typed key or value.
        
        A discussion of the use of dynamically typesafe views may be
        found in the documentation for the .checkedCollection
        checkedCollection method.
        
        The returned map will be serializable if the specified map is
        serializable.
        
        Since `null` is considered to be a value of any reference
        type, the returned map permits insertion of null keys or values
        whenever the backing map does.
        
        Type `<K>`: type of map keys
        
        Type `<V>`: type of map values

        Arguments
        - m: the map for which a dynamically typesafe view is to be
                 returned
        - keyType: the type of key that `m` is permitted to hold
        - valueType: the type of value that `m` is permitted to hold

        Returns
        - a dynamically typesafe view of the specified map

        Since
        - 1.8
        """
        ...


    @staticmethod
    def emptyIterator() -> Iterator["T"]:
        """
        Returns an iterator that has no elements.  More precisely,
        
        
        - Iterator.hasNext hasNext always returns `False`.
        - Iterator.next next always throws NoSuchElementException.
        - Iterator.remove remove always throws IllegalStateException.
        
        
        Implementations of this method are permitted, but not
        required, to return the same object from multiple invocations.
        
        Type `<T>`: type of elements, if there were any, in the iterator

        Returns
        - an empty iterator

        Since
        - 1.7
        """
        ...


    @staticmethod
    def emptyListIterator() -> "ListIterator"["T"]:
        """
        Returns a list iterator that has no elements.  More precisely,
        
        
        - Iterator.hasNext hasNext and ListIterator.hasPrevious hasPrevious always return `False`.
        - Iterator.next next and ListIterator.previous
        previous always throw NoSuchElementException.
        - Iterator.remove remove and ListIterator.set
        set always throw IllegalStateException.
        - ListIterator.add add always throws UnsupportedOperationException.
        - ListIterator.nextIndex nextIndex always returns
        `0`.
        - ListIterator.previousIndex previousIndex always
        returns `-1`.
        
        
        Implementations of this method are permitted, but not
        required, to return the same object from multiple invocations.
        
        Type `<T>`: type of elements, if there were any, in the iterator

        Returns
        - an empty list iterator

        Since
        - 1.7
        """
        ...


    @staticmethod
    def emptyEnumeration() -> "Enumeration"["T"]:
        """
        Returns an enumeration that has no elements.  More precisely,
        
        
        - Enumeration.hasMoreElements hasMoreElements always
        returns `False`.
        -  Enumeration.nextElement nextElement always throws
        NoSuchElementException.
        
        
        Implementations of this method are permitted, but not
        required, to return the same object from multiple invocations.
        
        Type `<T>`: the class of the objects in the enumeration

        Returns
        - an empty enumeration

        Since
        - 1.7
        """
        ...


    @staticmethod
    def emptySet() -> set["T"]:
        """
        Returns an empty set (immutable).  This set is serializable.
        Unlike the like-named field, this method is parameterized.
        
        This example illustrates the type-safe way to obtain an empty set:
        ```
            Set&lt;String&gt; s = Collections.emptySet();
        ```
        
        Type `<T>`: the class of the objects in the set

        Returns
        - the empty set

        See
        - .EMPTY_SET

        Since
        - 1.5

        Unknown Tags
        - Implementations of this method need not create a separate
        `Set` object for each call.  Using this method is likely to have
        comparable cost to using the like-named field.  (Unlike this method, the
        field does not provide type safety.)
        """
        ...


    @staticmethod
    def emptySortedSet() -> "SortedSet"["E"]:
        """
        Returns an empty sorted set (immutable).  This set is serializable.
        
        This example illustrates the type-safe way to obtain an empty
        sorted set:
        ``` `SortedSet<String> s = Collections.emptySortedSet();````
        
        Type `<E>`: type of elements, if there were any, in the set

        Returns
        - the empty sorted set

        Since
        - 1.8

        Unknown Tags
        - Implementations of this method need not create a separate
        `SortedSet` object for each call.
        """
        ...


    @staticmethod
    def emptyNavigableSet() -> "NavigableSet"["E"]:
        """
        Returns an empty navigable set (immutable).  This set is serializable.
        
        This example illustrates the type-safe way to obtain an empty
        navigable set:
        ``` `NavigableSet<String> s = Collections.emptyNavigableSet();````
        
        Type `<E>`: type of elements, if there were any, in the set

        Returns
        - the empty navigable set

        Since
        - 1.8

        Unknown Tags
        - Implementations of this method need not
        create a separate `NavigableSet` object for each call.
        """
        ...


    @staticmethod
    def emptyList() -> list["T"]:
        """
        Returns an empty list (immutable).  This list is serializable.
        
        This example illustrates the type-safe way to obtain an empty list:
        ```
            List&lt;String&gt; s = Collections.emptyList();
        ```
        
        Type `<T>`: type of elements, if there were any, in the list

        Returns
        - an empty immutable list

        See
        - .EMPTY_LIST

        Since
        - 1.5

        Unknown Tags
        - Implementations of this method need not create a separate `List`
        object for each call.   Using this method is likely to have comparable
        cost to using the like-named field.  (Unlike this method, the field does
        not provide type safety.)
        """
        ...


    @staticmethod
    def emptyMap() -> dict["K", "V"]:
        """
        Returns an empty map (immutable).  This map is serializable.
        
        This example illustrates the type-safe way to obtain an empty map:
        ```
            Map&lt;String, Date&gt; s = Collections.emptyMap();
        ```
        
        Type `<K>`: the class of the map keys
        
        Type `<V>`: the class of the map values

        Returns
        - an empty map

        See
        - .EMPTY_MAP

        Since
        - 1.5

        Unknown Tags
        - Implementations of this method need not create a separate
        `Map` object for each call.  Using this method is likely to have
        comparable cost to using the like-named field.  (Unlike this method, the
        field does not provide type safety.)
        """
        ...


    @staticmethod
    def emptySortedMap() -> "SortedMap"["K", "V"]:
        """
        Returns an empty sorted map (immutable).  This map is serializable.
        
        This example illustrates the type-safe way to obtain an empty map:
        ``` `SortedMap<String, Date> s = Collections.emptySortedMap();````
        
        Type `<K>`: the class of the map keys
        
        Type `<V>`: the class of the map values

        Returns
        - an empty sorted map

        Since
        - 1.8

        Unknown Tags
        - Implementations of this method need not create a separate
        `SortedMap` object for each call.
        """
        ...


    @staticmethod
    def emptyNavigableMap() -> "NavigableMap"["K", "V"]:
        """
        Returns an empty navigable map (immutable).  This map is serializable.
        
        This example illustrates the type-safe way to obtain an empty map:
        ``` `NavigableMap<String, Date> s = Collections.emptyNavigableMap();````
        
        Type `<K>`: the class of the map keys
        
        Type `<V>`: the class of the map values

        Returns
        - an empty navigable map

        Since
        - 1.8

        Unknown Tags
        - Implementations of this method need not create a separate
        `NavigableMap` object for each call.
        """
        ...


    @staticmethod
    def singleton(o: "T") -> set["T"]:
        """
        Returns an immutable set containing only the specified object.
        The returned set is serializable.
        
        Type `<T>`: the class of the objects in the set

        Arguments
        - o: the sole object to be stored in the returned set.

        Returns
        - an immutable set containing only the specified object.
        """
        ...


    @staticmethod
    def singletonList(o: "T") -> list["T"]:
        """
        Returns an immutable list containing only the specified object.
        The returned list is serializable.
        
        Type `<T>`: the class of the objects in the list

        Arguments
        - o: the sole object to be stored in the returned list.

        Returns
        - an immutable list containing only the specified object.

        Since
        - 1.3
        """
        ...


    @staticmethod
    def singletonMap(key: "K", value: "V") -> dict["K", "V"]:
        """
        Returns an immutable map, mapping only the specified key to the
        specified value.  The returned map is serializable.
        
        Type `<K>`: the class of the map keys
        
        Type `<V>`: the class of the map values

        Arguments
        - key: the sole key to be stored in the returned map.
        - value: the value to which the returned map maps `key`.

        Returns
        - an immutable map containing only the specified key-value
                mapping.

        Since
        - 1.3
        """
        ...


    @staticmethod
    def nCopies(n: int, o: "T") -> list["T"]:
        """
        Returns an immutable list consisting of `n` copies of the
        specified object.  The newly allocated data object is tiny (it contains
        a single reference to the data object).  This method is useful in
        combination with the `List.addAll` method to grow lists.
        The returned list is serializable.
        
        Type `<T>`: the class of the object to copy and of the objects
                in the returned list.

        Arguments
        - n: the number of elements in the returned list.
        - o: the element to appear repeatedly in the returned list.

        Returns
        - an immutable list consisting of `n` copies of the
                specified object.

        Raises
        - IllegalArgumentException: if `n < 0`

        See
        - List.addAll(int, Collection)
        """
        ...


    @staticmethod
    def reverseOrder() -> "Comparator"["T"]:
        """
        Returns a comparator that imposes the reverse of the *natural
        ordering* on a collection of objects that implement the
        `Comparable` interface.  (The natural ordering is the ordering
        imposed by the objects' own `compareTo` method.)  This enables a
        simple idiom for sorting (or maintaining) collections (or arrays) of
        objects that implement the `Comparable` interface in
        reverse-natural-order.  For example, suppose `a` is an array of
        strings. Then: ```
                 Arrays.sort(a, Collections.reverseOrder());
        ``` sorts the array in reverse-lexicographic (alphabetical) order.
        
        The returned comparator is serializable.
        
        Type `<T>`: the class of the objects compared by the comparator

        Returns
        - A comparator that imposes the reverse of the *natural
                ordering* on a collection of objects that implement
                the `Comparable` interface.

        See
        - Comparable
        """
        ...


    @staticmethod
    def reverseOrder(cmp: "Comparator"["T"]) -> "Comparator"["T"]:
        """
        Returns a comparator that imposes the reverse ordering of the specified
        comparator.  If the specified comparator is `null`, this method is
        equivalent to .reverseOrder() (in other words, it returns a
        comparator that imposes the reverse of the *natural ordering* on
        a collection of objects that implement the Comparable interface).
        
        The returned comparator is serializable (assuming the specified
        comparator is also serializable or `null`).
        
        Type `<T>`: the class of the objects compared by the comparator

        Arguments
        - cmp: a comparator who's ordering is to be reversed by the returned
        comparator or `null`

        Returns
        - A comparator that imposes the reverse ordering of the
                specified comparator.

        Since
        - 1.5
        """
        ...


    @staticmethod
    def enumeration(c: Iterable["T"]) -> "Enumeration"["T"]:
        """
        Returns an enumeration over the specified collection.  This provides
        interoperability with legacy APIs that require an enumeration
        as input.
        
        The iterator returned from a call to Enumeration.asIterator()
        does not support removal of elements from the specified collection.  This
        is necessary to avoid unintentionally increasing the capabilities of the
        returned enumeration.
        
        Type `<T>`: the class of the objects in the collection

        Arguments
        - c: the collection for which an enumeration is to be returned.

        Returns
        - an enumeration over the specified collection.

        See
        - Enumeration
        """
        ...


    @staticmethod
    def list(e: "Enumeration"["T"]) -> list["T"]:
        """
        Returns an array list containing the elements returned by the
        specified enumeration in the order they are returned by the
        enumeration.  This method provides interoperability between
        legacy APIs that return enumerations and new APIs that require
        collections.
        
        Type `<T>`: the class of the objects returned by the enumeration

        Arguments
        - e: enumeration providing elements for the returned
                 array list

        Returns
        - an array list containing the elements returned
                by the specified enumeration.

        See
        - ArrayList

        Since
        - 1.4
        """
        ...


    @staticmethod
    def frequency(c: Iterable[Any], o: "Object") -> int:
        """
        Returns the number of elements in the specified collection equal to the
        specified object.  More formally, returns the number of elements
        `e` in the collection such that
        `Objects.equals(o, e)`.

        Arguments
        - c: the collection in which to determine the frequency
            of `o`
        - o: the object whose frequency is to be determined

        Returns
        - the number of elements in `c` equal to `o`

        Raises
        - NullPointerException: if `c` is null

        Since
        - 1.5
        """
        ...


    @staticmethod
    def disjoint(c1: Iterable[Any], c2: Iterable[Any]) -> bool:
        """
        Returns `True` if the two specified collections have no
        elements in common.
        
        Care must be exercised if this method is used on collections that
        do not comply with the general contract for `Collection`.
        Implementations may elect to iterate over either collection and test
        for containment in the other collection (or to perform any equivalent
        computation).  If either collection uses a nonstandard equality test
        (as does a SortedSet whose ordering is not *compatible with
        equals*, or the key set of an IdentityHashMap), both
        collections must use the same nonstandard equality test, or the
        result of this method is undefined.
        
        Care must also be exercised when using collections that have
        restrictions on the elements that they may contain. Collection
        implementations are allowed to throw exceptions for any operation
        involving elements they deem ineligible. For absolute safety the
        specified collections should contain only elements which are
        eligible elements for both collections.
        
        Note that it is permissible to pass the same collection in both
        parameters, in which case the method will return `True` if and
        only if the collection is empty.

        Arguments
        - c1: a collection
        - c2: a collection

        Returns
        - `True` if the two specified collections have no
        elements in common.

        Raises
        - NullPointerException: if either collection is `null`.
        - NullPointerException: if one collection contains a `null`
        element and `null` is not an eligible element for the other collection.
        (<a href="Collection.html#optional-restrictions">optional</a>)
        - ClassCastException: if one collection contains an element that is
        of a type which is ineligible for the other collection.
        (<a href="Collection.html#optional-restrictions">optional</a>)

        Since
        - 1.5
        """
        ...


    @staticmethod
    def addAll(c: Iterable["T"], *elements: Tuple["T", ...]) -> bool:
        """
        Adds all of the specified elements to the specified collection.
        Elements to be added may be specified individually or as an array.
        The behaviour of this convenience method is similar to that of
        `cc.addAll(Collections.unmodifiableList(Arrays.asList(elements)))`.
        
        When elements are specified individually, this method provides a
        convenient way to add a few elements to an existing collection:
        ```
            Collections.addAll(flavors, "Peaches 'n Plutonium", "Rocky Racoon");
        ```
        
        Type `<T>`: the class of the elements to add and of the collection

        Arguments
        - c: the collection into which `elements` are to be inserted
        - elements: the elements to insert into `c`

        Returns
        - `True` if the collection changed as a result of the call

        Raises
        - UnsupportedOperationException: if `c` does not support
                the `add` operation
        - NullPointerException: if `elements` contains one or more
                null values and `c` does not permit null elements, or
                if `c` or `elements` are `null`
        - IllegalArgumentException: if some property of a value in
                `elements` prevents it from being added to `c`

        See
        - Collection.addAll(Collection)

        Since
        - 1.5
        """
        ...


    @staticmethod
    def newSetFromMap(map: dict["E", "Boolean"]) -> set["E"]:
        """
        Returns a set backed by the specified map.  The resulting set displays
        the same ordering, concurrency, and performance characteristics as the
        backing map.  In essence, this factory method provides a Set
        implementation corresponding to any Map implementation.  There
        is no need to use this method on a Map implementation that
        already has a corresponding Set implementation (such as HashMap or TreeMap).
        
        Each method invocation on the set returned by this method results in
        exactly one method invocation on the backing map or its `keySet`
        view, with one exception.  The `addAll` method is implemented
        as a sequence of `put` invocations on the backing map.
        
        The specified map must be empty at the time this method is invoked,
        and should not be accessed directly after this method returns.  These
        conditions are ensured if the map is created empty, passed directly
        to this method, and no reference to the map is retained, as illustrated
        in the following code fragment:
        ```
           Set&lt;Object&gt; weakHashSet = Collections.newSetFromMap(
               new WeakHashMap&lt;Object, Boolean&gt;());
        ```
        
        Type `<E>`: the class of the map keys and of the objects in the
               returned set

        Arguments
        - map: the backing map

        Returns
        - the set backed by the map

        Raises
        - IllegalArgumentException: if `map` is not empty

        Since
        - 1.6
        """
        ...


    @staticmethod
    def asLifoQueue(deque: "Deque"["T"]) -> "Queue"["T"]:
        """
        Returns a view of a Deque as a Last-in-first-out (Lifo)
        Queue. Method `add` is mapped to `push`,
        `remove` is mapped to `pop` and so on. This
        view can be useful when you would like to use a method
        requiring a `Queue` but you need Lifo ordering.
        
        Each method invocation on the queue returned by this method
        results in exactly one method invocation on the backing deque, with
        one exception.  The Queue.addAll addAll method is
        implemented as a sequence of Deque.addFirst addFirst
        invocations on the backing deque.
        
        Type `<T>`: the class of the objects in the deque

        Arguments
        - deque: the deque

        Returns
        - the queue

        Since
        - 1.6
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.data.type.StructureBlock

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.block.data import BlockData
from org.bukkit.block.data.type import *
from typing import Any, Callable, Iterable, Tuple


class StructureBlock(BlockData):
    """
    'mode' represents the different modes in which this structure block may
    operate.
    """

    def getMode(self) -> "Mode":
        """
        Gets the value of the 'mode' property.

        Returns
        - the 'mode' value
        """
        ...


    def setMode(self, mode: "Mode") -> None:
        """
        Sets the value of the 'mode' property.

        Arguments
        - mode: the new 'mode' value
        """
        ...


    class Mode(Enum):
        """
        Operating mode of a structure block.
        """

        SAVE = 0
        """
        Allows selection and saving of a structure.
        """
        LOAD = 1
        """
        Allows loading of a structure.
        """
        CORNER = 2
        """
        Used for detection of two opposite corners of a structure.
        """
        DATA = 3
        """
        Dummy block used to run a custom function during world generation
        before being removed.
        """



"""
Python module generated from Java source file org.bukkit.event.entity.VillagerCareerChangeEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.entity import Villager
from org.bukkit.entity.Villager import Profession
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.entity import *
from typing import Any, Callable, Iterable, Tuple


class VillagerCareerChangeEvent(EntityEvent, Cancellable):

    def __init__(self, what: "Villager", profession: "Profession", reason: "ChangeReason"):
        ...


    def getEntity(self) -> "Villager":
        ...


    def getProfession(self) -> "Profession":
        """
        Gets the future profession of the villager.

        Returns
        - The profession the villager will change to
        """
        ...


    def setProfession(self, profession: "Profession") -> None:
        """
        Sets the profession the villager will become from this event.

        Arguments
        - profession: new profession
        """
        ...


    def getReason(self) -> "ChangeReason":
        """
        Gets the reason for why the villager's career is changing.

        Returns
        - Reason for villager's profession changing
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...


    class ChangeReason(Enum):
        """
        Reasons for the villager's profession changing.
        """

        LOSING_JOB = 0
        """
        Villager lost their job due to too little experience.
        """
        EMPLOYED = 1
        """
        Villager gained employment.
        """



"""
Python module generated from Java source file org.bukkit.event.weather.WeatherEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import World
from org.bukkit.event import Event
from org.bukkit.event.weather import *
from typing import Any, Callable, Iterable, Tuple


class WeatherEvent(Event):
    """
    Represents a Weather-related event
    """

    def __init__(self, where: "World"):
        ...


    def getWorld(self) -> "World":
        """
        Returns the World where this event is occurring

        Returns
        - World this event is occurring in
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.entity.CreatureSpawnEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit import Chunk
from org.bukkit.entity import LivingEntity
from org.bukkit.event.entity import *
from org.bukkit.event.world import ChunkLoadEvent
from typing import Any, Callable, Iterable, Tuple


class CreatureSpawnEvent(EntitySpawnEvent):
    """
    Called when a creature is spawned into a world.
    
    If a Creature Spawn event is cancelled, the creature will not spawn.
    """

    def __init__(self, spawnee: "LivingEntity", spawnReason: "SpawnReason"):
        ...


    def getEntity(self) -> "LivingEntity":
        ...


    def getSpawnReason(self) -> "SpawnReason":
        """
        Gets the reason for why the creature is being spawned.

        Returns
        - A SpawnReason value detailing the reason for the creature being
            spawned
        """
        ...


    class SpawnReason(Enum):
        """
        An enum to specify the type of spawning
        """

        NATURAL = 0
        """
        When something spawns from natural means
        """
        JOCKEY = 1
        """
        When an entity spawns as a jockey of another entity (mostly spider
        jockeys)
        """
        CHUNK_GEN = 2
        """
        When a creature spawns due to chunk generation

        Deprecated
        - no longer called, chunks are generated with entities
        already existing. Consider using ChunkLoadEvent,
        ChunkLoadEvent.isNewChunk() and Chunk.getEntities()
        for similar effect.
        """
        SPAWNER = 3
        """
        When a creature spawns from a spawner
        """
        TRIAL_SPAWNER = 4
        """
        When a creature spawns from a trial spawner
        """
        EGG = 5
        """
        When a creature spawns from an egg
        """
        SPAWNER_EGG = 6
        """
        When a creature spawns from a Spawner Egg
        """
        LIGHTNING = 7
        """
        When a creature spawns because of a lightning strike
        """
        BUILD_SNOWMAN = 8
        """
        When a snowman is spawned by being built
        """
        BUILD_IRONGOLEM = 9
        """
        When an iron golem is spawned by being built
        """
        BUILD_WITHER = 10
        """
        When a wither boss is spawned by being built
        """
        VILLAGE_DEFENSE = 11
        """
        When an iron golem is spawned to defend a village
        """
        VILLAGE_INVASION = 12
        """
        When a zombie is spawned to invade a village
        """
        BREEDING = 13
        """
        When an entity breeds to create a child, this also include Shulker and Allay
        """
        SLIME_SPLIT = 14
        """
        When a slime splits
        """
        REINFORCEMENTS = 15
        """
        When an entity calls for reinforcements
        """
        NETHER_PORTAL = 16
        """
        When a creature is spawned by nether portal
        """
        DISPENSE_EGG = 17
        """
        When a creature is spawned by a dispenser dispensing an egg
        """
        INFECTION = 18
        """
        When a zombie infects a villager
        """
        CURED = 19
        """
        When a villager is cured from infection
        """
        OCELOT_BABY = 20
        """
        When an ocelot has a baby spawned along with them
        """
        SILVERFISH_BLOCK = 21
        """
        When a silverfish spawns from a block
        """
        MOUNT = 22
        """
        When an entity spawns as a mount of another entity (mostly chicken
        jockeys)
        """
        TRAP = 23
        """
        When an entity spawns as a trap for players approaching
        """
        ENDER_PEARL = 24
        """
        When an entity is spawned as a result of ender pearl usage
        """
        SHOULDER_ENTITY = 25
        """
        When an entity is spawned as a result of the entity it is being
        perched on jumping or being damaged
        """
        DROWNED = 26
        """
        When a creature is spawned by another entity drowning
        """
        SHEARED = 27
        """
        When an cow is spawned by shearing a mushroom cow
        """
        EXPLOSION = 28
        """
        When eg an effect cloud is spawned as a result of a creeper exploding
        """
        RAID = 29
        """
        When an entity is spawned as part of a raid
        """
        PATROL = 30
        """
        When an entity is spawned as part of a patrol
        """
        BEEHIVE = 31
        """
        When a bee is released from a beehive/bee nest
        """
        PIGLIN_ZOMBIFIED = 32
        """
        When a piglin is converted to a zombified piglin.
        """
        SPELL = 33
        """
        When an entity is created by a cast spell.
        """
        FROZEN = 34
        """
        When an entity is shaking in Powder Snow and a new entity spawns.
        """
        METAMORPHOSIS = 35
        """
        When a tadpole converts to a frog
        """
        DUPLICATION = 36
        """
        When an Allay duplicate itself
        """
        COMMAND = 37
        """
        When a creature is spawned by the "/summon" command
        """
        ENCHANTMENT = 38
        """
        When a creature is spawned by an enchantment
        """
        POTION_EFFECT = 39
        """
        When a creature is spawned by a potion effect, for example:
        org.bukkit.potion.PotionType.OOZING, org.bukkit.potion.PotionType.INFESTED
        """
        CUSTOM = 40
        """
        When a creature is spawned by plugins
        """
        DEFAULT = 41
        """
        When an entity is missing a SpawnReason
        """



"""
Python module generated from Java source file org.bukkit.persistence.ListPersistentDataType

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.persistence import *
from typing import Any, Callable, Iterable, Tuple


class ListPersistentDataType(PersistentDataType):
    """
    The list persistent data represents a data type that is capable of storing a
    list of other data types in a PersistentDataContainer.
    
    Type `<P>`: the primitive type of the list element.
    
    Type `<C>`: the complex type of the list elements.
    """

    def elementType(self) -> "PersistentDataType"["P", "C"]:
        """
        Provides the persistent data type of the elements found in the list.

        Returns
        - the persistent data type.
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerMoveEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from org.bukkit import Location
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from typing import Any, Callable, Iterable, Tuple


class PlayerMoveEvent(PlayerEvent, Cancellable):
    """
    Holds information for player movement events
    """

    def __init__(self, player: "Player", from: "Location", to: "Location"):
        ...


    def isCancelled(self) -> bool:
        """
        Gets the cancellation state of this event. A cancelled event will not
        be executed in the server, but will still pass to other plugins
        
        If a move or teleport event is cancelled, the player will be moved or
        teleported back to the Location as defined by getFrom(). This will not
        fire an event

        Returns
        - True if this event is cancelled
        """
        ...


    def setCancelled(self, cancel: bool) -> None:
        """
        Sets the cancellation state of this event. A cancelled event will not
        be executed in the server, but will still pass to other plugins
        
        If a move or teleport event is cancelled, the player will be moved or
        teleported back to the Location as defined by getFrom(). This will not
        fire an event

        Arguments
        - cancel: True if you wish to cancel this event
        """
        ...


    def getFrom(self) -> "Location":
        """
        Gets the location this player moved from

        Returns
        - Location the player moved from
        """
        ...


    def setFrom(self, from: "Location") -> None:
        """
        Sets the location to mark as where the player moved from

        Arguments
        - from: New location to mark as the players previous location
        """
        ...


    def getTo(self) -> "Location":
        """
        Gets the location this player moved to

        Returns
        - Location the player moved to
        """
        ...


    def setTo(self, to: "Location") -> None:
        """
        Sets the location that this player will move to

        Arguments
        - to: New Location this player will move to
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file java.nio.file.DirectoryIteratorException

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import IOException
from java.io import InvalidObjectException
from java.io import ObjectInputStream
from java.nio.file import *
from java.util import ConcurrentModificationException
from java.util import Objects
from typing import Any, Callable, Iterable, Tuple


class DirectoryIteratorException(ConcurrentModificationException):

    def __init__(self, cause: "IOException"):
        """
        Constructs an instance of this class.

        Arguments
        - cause: the `IOException` that caused the directory iteration
                 to fail

        Raises
        - NullPointerException: if the cause is `null`
        """
        ...


    def getCause(self) -> "IOException":
        """
        Returns the cause of this exception.

        Returns
        - the cause
        """
        ...



from .PerlinNoiseGenerator import PerlinNoiseGenerator
from .PerlinOctaveGenerator import PerlinOctaveGenerator
from .SimplexNoiseGenerator import SimplexNoiseGenerator
from .NoiseGenerator import NoiseGenerator
from .SimplexOctaveGenerator import SimplexOctaveGenerator
from .OctaveGenerator import OctaveGenerator


"""
Python module generated from Java source file com.google.common.collect.Cut

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from java.io import Serializable
from java.util import NoSuchElementException
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class Cut(Comparable, Serializable):
    """
    Implementation detail for the internal structure of Range instances. Represents a unique
    way of "cutting" a "number line" (actually of instances of type `C`, not necessarily
    "numbers") into two sections; this can be done below a certain value, above a certain value,
    below all values or above all values. With this object defined in this way, an interval can
    always be represented by a pair of `Cut` instances.

    Author(s)
    - Kevin Bourrillion
    """

    def compareTo(self, that: "Cut"["C"]) -> int:
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...



"""
Python module generated from Java source file com.google.common.base.Converter

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import *
from com.google.errorprone.annotations import CheckReturnValue
from com.google.errorprone.annotations import ForOverride
from com.google.errorprone.annotations import InlineMe
from com.google.errorprone.annotations.concurrent import LazyInit
from com.google.j2objc.annotations import RetainedWith
from java.io import Serializable
from java.util import Iterator
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class Converter(Function):
    """
    A function from `A` to `B` with an associated *reverse* function from `B`
    to `A`; used for converting back and forth between *different representations of the same
    information*.
    
    <h3>Invertibility</h3>
    
    The reverse operation **may** be a strict *inverse* (meaning that `converter.reverse().convert(converter.convert(a)).equals(a)` is always True). However, it is very
    common (perhaps *more* common) for round-trip conversion to be *lossy*. Consider an
    example round-trip using com.google.common.primitives.Doubles.stringConverter:
    
    <ol>
      - `stringConverter().convert("1.00")` returns the `Double` value `1.0`
      - `stringConverter().reverse().convert(1.0)` returns the string `"1.0"` --
          *not* the same string (`"1.00"`) we started with
    </ol>
    
    Note that it should still be the case that the round-tripped and original objects are
    *similar*.
    
    <h3>Nullability</h3>
    
    A converter always converts `null` to `null` and non-null references to non-null
    references. It would not make sense to consider `null` and a non-null reference to be
    "different representations of the same information", since one is distinguishable from
    *missing* information and the other is not. The .convert method handles this null
    behavior for all converters; implementations of .doForward and .doBackward are
    guaranteed to never be passed `null`, and must never return `null`.
    
    <h3>Common ways to use</h3>
    
    Getting a converter:
    
    
      - Use a provided converter implementation, such as Enums.stringConverter, com.google.common.primitives.Ints.stringConverter Ints.stringConverter or the .reverse reverse views of these.
      - Convert between specific preset values using com.google.common.collect.Maps.asConverter Maps.asConverter. For example, use this to
          create a "fake" converter for a unit test. It is unnecessary (and confusing) to *mock*
          the `Converter` type using a mocking framework.
      - Extend this class and implement its .doForward and .doBackward methods.
      - **Java 8+ users:** you may prefer to pass two lambda expressions or method references to
          the .from from factory method.
    
    
    Using a converter:
    
    
      - Convert one instance in the "forward" direction using `converter.convert(a)`.
      - Convert multiple instances "forward" using `converter.convertAll(as)`.
      - Convert in the "backward" direction using `converter.reverse().convert(b)` or `converter.reverse().convertAll(bs)`.
      - Use `converter` or `converter.reverse()` anywhere a java.util.function.Function is accepted (for example java.util.stream.Stream.map
          Stream.map).
      - **Do not** call .doForward or .doBackward directly; these exist only to
          be overridden.
    
    
    <h3>Example</h3>
    
    ```
      return new Converter&lt;Integer, String&gt;() {
        protected String doForward(Integer i) {
          return Integer.toHexString(i);
        }
    
        protected Integer doBackward(String s) {
          return parseUnsignedInt(s, 16);
        }
      };```
    
    An alternative using Java 8:
    
    ````return Converter.from(
        Integer::toHexString,
        s -> parseUnsignedInt(s, 16));````

    Author(s)
    - Gregory Kick

    Since
    - 16.0
    """

    def convert(self, a: "A") -> "B":
        """
        Returns a representation of `a` as an instance of type `B`.

        Returns
        - the converted value; is null *if and only if* `a` is null
        """
        ...


    def convertAll(self, fromIterable: Iterable["A"]) -> Iterable["B"]:
        ...


    def reverse(self) -> "Converter"["B", "A"]:
        """
        Returns the reversed view of this converter, which converts `this.convert(a)` back to a
        value roughly equivalent to `a`.
        
        The returned converter is serializable if `this` converter is.
        
        **Note:** you should not override this method. It is non-final for legacy reasons.
        """
        ...


    def andThen(self, secondConverter: "Converter"["B", "C"]) -> "Converter"["A", "C"]:
        """
        Returns a converter whose `convert` method applies `secondConverter` to the result
        of this converter. Its `reverse` method applies the converters in reverse order.
        
        The returned converter is serializable if `this` converter and `secondConverter`
        are.
        """
        ...


    def apply(self, a: "A") -> "B":
        """
        Deprecated
        - Provided to satisfy the `Function` interface; use .convert instead.
        """
        ...


    def equals(self, object: "Object") -> bool:
        """
        Indicates whether another object is equal to this converter.
        
        Most implementations will have no reason to override the behavior of Object.equals.
        However, an implementation may also choose to return `True` whenever `object` is a
        Converter that it considers *interchangeable* with this one. "Interchangeable"
        *typically* means that `Objects.equal(this.convert(a), that.convert(a))` is True for
        all `a` of type `A` (and similarly for `reverse`). Note that a `False`
        result from this method does not imply that the converters are known *not* to be
        interchangeable.
        """
        ...


    @staticmethod
    def from(forwardFunction: "Function"["A", "B"], backwardFunction: "Function"["B", "A"]) -> "Converter"["A", "B"]:
        """
        Returns a converter based on separate forward and backward functions. This is useful if the
        function instances already exist, or so that you can supply lambda expressions. If those
        circumstances don't apply, you probably don't need to use this; subclass `Converter` and
        implement its .doForward and .doBackward methods directly.
        
        These functions will never be passed `null` and must not under any circumstances
        return `null`. If a value cannot be converted, the function should throw an unchecked
        exception (typically, but not necessarily, IllegalArgumentException).
        
        The returned converter is serializable if both provided functions are.

        Since
        - 17.0
        """
        ...


    @staticmethod
    def identity() -> "Converter"["T", "T"]:
        """
        Returns a serializable converter that always converts or reverses an object to itself.
        """
        ...



"""
Python module generated from Java source file com.google.common.hash.FarmHashFingerprint64

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import VisibleForTesting
from com.google.common.hash import *
from typing import Any, Callable, Iterable, Tuple


class FarmHashFingerprint64(AbstractNonStreamingHashFunction):
    """
    Implementation of FarmHash Fingerprint64, an open-source fingerprinting algorithm for strings.
    
    Its speed is comparable to CityHash64, and its quality of hashing is at least as good.
    
    Note to maintainers: This implementation relies on signed arithmetic being bit-wise equivalent
    to unsigned arithmetic in all cases except:
    
    
      - comparisons (signed values can be negative)
      - division (avoided here)
      - shifting (right shift must be unsigned)

    Author(s)
    - Geoff Pike
    """

    def hashBytes(self, input: list[int], off: int, len: int) -> "HashCode":
        ...


    def bits(self) -> int:
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file org.bukkit.conversations.ExactMatchConversationCanceller

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.conversations import *
from typing import Any, Callable, Iterable, Tuple


class ExactMatchConversationCanceller(ConversationCanceller):
    """
    An ExactMatchConversationCanceller cancels a conversation if the user
    enters an exact input string
    """

    def __init__(self, escapeSequence: str):
        """
        Builds an ExactMatchConversationCanceller.

        Arguments
        - escapeSequence: The string that, if entered by the user, will
            cancel the conversation.
        """
        ...


    def setConversation(self, conversation: "Conversation") -> None:
        ...


    def cancelBasedOnInput(self, context: "ConversationContext", input: str) -> bool:
        ...


    def clone(self) -> "ConversationCanceller":
        ...



"""
Python module generated from Java source file org.bukkit.entity.Cat

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from com.google.common.collect import Lists
from java.util import Locale
from org.bukkit import DyeColor
from org.bukkit import Keyed
from org.bukkit import NamespacedKey
from org.bukkit import Registry
from org.bukkit.entity import *
from org.bukkit.registry import RegistryAware
from org.bukkit.util import OldEnum
from typing import Any, Callable, Iterable, Tuple


class Cat(Tameable, Sittable):
    """
    Meow.
    """

    def getCatType(self) -> "Type":
        """
        Gets the current type of this cat.

        Returns
        - Type of the cat.
        """
        ...


    def setCatType(self, type: "Type") -> None:
        """
        Sets the current type of this cat.

        Arguments
        - type: New type of this cat.
        """
        ...


    def getCollarColor(self) -> "DyeColor":
        """
        Get the collar color of this cat

        Returns
        - the color of the collar
        """
        ...


    def setCollarColor(self, color: "DyeColor") -> None:
        """
        Set the collar color of this cat

        Arguments
        - color: the color to apply
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerPortalEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Location
from org.bukkit.entity import Player
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from typing import Any, Callable, Iterable, Tuple


class PlayerPortalEvent(PlayerTeleportEvent):
    """
    Called when a player is about to teleport because it is in contact with a
    portal which will generate an exit portal.
    
    For other entities see org.bukkit.event.entity.EntityPortalEvent
    """

    def __init__(self, player: "Player", from: "Location", to: "Location"):
        ...


    def __init__(self, player: "Player", from: "Location", to: "Location", cause: "TeleportCause"):
        ...


    def __init__(self, player: "Player", from: "Location", to: "Location", cause: "TeleportCause", getSearchRadius: int, canCreatePortal: bool, creationRadius: int):
        ...


    def setSearchRadius(self, searchRadius: int) -> None:
        """
        Set the Block radius to search in for available portals.

        Arguments
        - searchRadius: the radius in which to search for a portal from the
        location
        """
        ...


    def getSearchRadius(self) -> int:
        """
        Gets the search radius value for finding an available portal.

        Returns
        - the currently set search radius
        """
        ...


    def getCanCreatePortal(self) -> bool:
        """
        Returns whether the server will attempt to create a destination portal or
        not.

        Returns
        - whether there should create be a destination portal created
        """
        ...


    def setCanCreatePortal(self, canCreatePortal: bool) -> None:
        """
        Sets whether the server should attempt to create a destination portal or
        not.

        Arguments
        - canCreatePortal: Sets whether there should be a destination portal
        created
        """
        ...


    def setCreationRadius(self, creationRadius: int) -> None:
        """
        Sets the maximum radius the world is searched for a free space from the
        given location.
        
        If enough free space is found then the portal will be created there, if
        not it will force create with air-space at the target location.
        
        Does not apply to end portal target platforms which will always appear at
        the target location.

        Arguments
        - creationRadius: the radius in which to create a portal from the
        location
        """
        ...


    def getCreationRadius(self) -> int:
        """
        Gets the maximum radius the world is searched for a free space from the
        given location.
        
        If enough free space is found then the portal will be created there, if
        not it will force create with air-space at the target location.
        
        Does not apply to end portal target platforms which will always appear at
        the target location.

        Returns
        - the currently set creation radius
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.inventory.meta.CrossbowMeta

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.inventory import ItemStack
from org.bukkit.inventory.meta import *
from typing import Any, Callable, Iterable, Tuple


class CrossbowMeta(ItemMeta):

    def hasChargedProjectiles(self) -> bool:
        """
        Returns whether the item has any charged projectiles.

        Returns
        - whether charged projectiles are present
        """
        ...


    def getChargedProjectiles(self) -> list["ItemStack"]:
        """
        Returns an immutable list of the projectiles charged on this item.

        Returns
        - charged projectiles
        """
        ...


    def setChargedProjectiles(self, projectiles: list["ItemStack"]) -> None:
        """
        Sets the projectiles charged on this item.
        
        Removes all projectiles when given null.

        Arguments
        - projectiles: the projectiles to set

        Raises
        - IllegalArgumentException: if one of the projectiles is not an
        arrow or firework rocket
        """
        ...


    def addChargedProjectile(self, item: "ItemStack") -> None:
        """
        Adds a charged projectile to this item.

        Arguments
        - item: projectile

        Raises
        - IllegalArgumentException: if the projectile is not an arrow or
        firework rocket
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.Multisets

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import Objects
from com.google.common.base import Predicate
from com.google.common.base import Predicates
from com.google.common.collect import *
from com.google.common.collect.Multiset import Entry
from com.google.common.math import IntMath
from com.google.common.primitives import Ints
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations.concurrent import LazyInit
from java.io import Serializable
from java.util import Arrays
from java.util import Collections
from java.util import Comparator
from java.util import Iterator
from java.util import NoSuchElementException
from java.util import Spliterator
from java.util.function import Function
from java.util.function import Supplier
from java.util.function import ToIntFunction
from java.util.stream import Collector
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Multisets:
    """
    Provides static utility methods for creating and working with Multiset instances.
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/CollectionUtilitiesExplained#multisets">`Multisets`</a>.

    Author(s)
    - Louis Wasserman

    Since
    - 2.0
    """

    @staticmethod
    def toMultiset(elementFunction: "Function"["T", "E"], countFunction: "ToIntFunction"["T"], multisetSupplier: "Supplier"["M"]) -> "Collector"["T", Any, "M"]:
        """
        Returns a `Collector` that accumulates elements into a multiset created via the specified
        `Supplier`, whose elements are the result of applying `elementFunction` to the
        inputs, with counts equal to the result of applying `countFunction` to the inputs.
        Elements are added in encounter order.
        
        If the mapped elements contain duplicates (according to Object.equals), the element
        will be added more than once, with the count summed over all appearances of the element.
        
        Note that `stream.collect(toMultiset(function, e -> 1, supplier))` is equivalent to
        `stream.map(function).collect(Collectors.toCollection(supplier))`.
        
        To collect to an ImmutableMultiset, use ImmutableMultiset.toImmutableMultiset.

        Since
        - 22.0
        """
        ...


    @staticmethod
    def unmodifiableMultiset(multiset: "Multiset"["E"]) -> "Multiset"["E"]:
        """
        Returns an unmodifiable view of the specified multiset. Query operations on the returned
        multiset "read through" to the specified multiset, and attempts to modify the returned multiset
        result in an UnsupportedOperationException.
        
        The returned multiset will be serializable if the specified multiset is serializable.

        Arguments
        - multiset: the multiset for which an unmodifiable view is to be generated

        Returns
        - an unmodifiable view of the multiset
        """
        ...


    @staticmethod
    def unmodifiableMultiset(multiset: "ImmutableMultiset"["E"]) -> "Multiset"["E"]:
        """
        Simply returns its argument.

        Since
        - 10.0

        Deprecated
        - no need to use this
        """
        ...


    @staticmethod
    def unmodifiableSortedMultiset(sortedMultiset: "SortedMultiset"["E"]) -> "SortedMultiset"["E"]:
        """
        Returns an unmodifiable view of the specified sorted multiset. Query operations on the returned
        multiset "read through" to the specified multiset, and attempts to modify the returned multiset
        result in an UnsupportedOperationException.
        
        The returned multiset will be serializable if the specified multiset is serializable.

        Arguments
        - sortedMultiset: the sorted multiset for which an unmodifiable view is to be generated

        Returns
        - an unmodifiable view of the multiset

        Since
        - 11.0
        """
        ...


    @staticmethod
    def immutableEntry(e: "E", n: int) -> "Multiset.Entry"["E"]:
        """
        Returns an immutable multiset entry with the specified element and count. The entry will be
        serializable if `e` is.

        Arguments
        - e: the element to be associated with the returned entry
        - n: the count to be associated with the returned entry

        Raises
        - IllegalArgumentException: if `n` is negative
        """
        ...


    @staticmethod
    def filter(unfiltered: "Multiset"["E"], predicate: "Predicate"["E"]) -> "Multiset"["E"]:
        """
        Returns a view of the elements of `unfiltered` that satisfy a predicate. The returned
        multiset is a live view of `unfiltered`; changes to one affect the other.
        
        The resulting multiset's iterators, and those of its `entrySet()` and `elementSet()`, do not support `remove()`. However, all other multiset methods supported
        by `unfiltered` are supported by the returned multiset. When given an element that
        doesn't satisfy the predicate, the multiset's `add()` and `addAll()` methods throw
        an IllegalArgumentException. When methods such as `removeAll()` and `clear()` are called on the filtered multiset, only elements that satisfy the filter will be
        removed from the underlying multiset.
        
        The returned multiset isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered multiset's methods, such as `size()`, iterate across every
        element in the underlying multiset and determine which elements satisfy the filter. When a live
        view is *not* needed, it may be faster to copy the returned multiset and use the copy.
        
        **Warning:** `predicate` must be *consistent with equals*, as documented at
        Predicate.apply. Do not provide a predicate such as `Predicates.instanceOf(ArrayList.class)`, which is inconsistent with equals. (See Iterables.filter(Iterable, Class) for related functionality.)

        Since
        - 14.0
        """
        ...


    @staticmethod
    def union(multiset1: "Multiset"["E"], multiset2: "Multiset"["E"]) -> "Multiset"["E"]:
        """
        Returns an unmodifiable view of the union of two multisets. In the returned multiset, the count
        of each element is the *maximum* of its counts in the two backing multisets. The iteration
        order of the returned multiset matches that of the element set of `multiset1` followed by
        the members of the element set of `multiset2` that are not contained in `multiset1`, with repeated occurrences of the same element appearing consecutively.
        
        Results are undefined if `multiset1` and `multiset2` are based on different
        equivalence relations (as `HashMultiset` and `TreeMultiset` are).

        Since
        - 14.0
        """
        ...


    @staticmethod
    def intersection(multiset1: "Multiset"["E"], multiset2: "Multiset"[Any]) -> "Multiset"["E"]:
        """
        Returns an unmodifiable view of the intersection of two multisets. In the returned multiset,
        the count of each element is the *minimum* of its counts in the two backing multisets,
        with elements that would have a count of 0 not included. The iteration order of the returned
        multiset matches that of the element set of `multiset1`, with repeated occurrences of the
        same element appearing consecutively.
        
        Results are undefined if `multiset1` and `multiset2` are based on different
        equivalence relations (as `HashMultiset` and `TreeMultiset` are).

        Since
        - 2.0
        """
        ...


    @staticmethod
    def sum(multiset1: "Multiset"["E"], multiset2: "Multiset"["E"]) -> "Multiset"["E"]:
        """
        Returns an unmodifiable view of the sum of two multisets. In the returned multiset, the count
        of each element is the *sum* of its counts in the two backing multisets. The iteration
        order of the returned multiset matches that of the element set of `multiset1` followed by
        the members of the element set of `multiset2` that are not contained in `multiset1`, with repeated occurrences of the same element appearing consecutively.
        
        Results are undefined if `multiset1` and `multiset2` are based on different
        equivalence relations (as `HashMultiset` and `TreeMultiset` are).

        Since
        - 14.0
        """
        ...


    @staticmethod
    def difference(multiset1: "Multiset"["E"], multiset2: "Multiset"[Any]) -> "Multiset"["E"]:
        """
        Returns an unmodifiable view of the difference of two multisets. In the returned multiset, the
        count of each element is the result of the *zero-truncated subtraction* of its count in
        the second multiset from its count in the first multiset, with elements that would have a count
        of 0 not included. The iteration order of the returned multiset matches that of the element set
        of `multiset1`, with repeated occurrences of the same element appearing consecutively.
        
        Results are undefined if `multiset1` and `multiset2` are based on different
        equivalence relations (as `HashMultiset` and `TreeMultiset` are).

        Since
        - 14.0
        """
        ...


    @staticmethod
    def containsOccurrences(superMultiset: "Multiset"[Any], subMultiset: "Multiset"[Any]) -> bool:
        """
        Returns `True` if `subMultiset.count(o) <= superMultiset.count(o)` for all `o`.

        Since
        - 10.0
        """
        ...


    @staticmethod
    def retainOccurrences(multisetToModify: "Multiset"[Any], multisetToRetain: "Multiset"[Any]) -> bool:
        """
        Modifies `multisetToModify` so that its count for an element `e` is at most `multisetToRetain.count(e)`.
        
        To be precise, `multisetToModify.count(e)` is set to `Math.min(multisetToModify.count(e), multisetToRetain.count(e))`. This is similar to .intersection(Multiset, Multiset) intersection `(multisetToModify, multisetToRetain)`,
        but mutates `multisetToModify` instead of returning a view.
        
        In contrast, `multisetToModify.retainAll(multisetToRetain)` keeps all occurrences of
        elements that appear at all in `multisetToRetain`, and deletes all occurrences of all
        other elements.

        Returns
        - `True` if `multisetToModify` was changed as a result of this operation

        Since
        - 10.0
        """
        ...


    @staticmethod
    def removeOccurrences(multisetToModify: "Multiset"[Any], occurrencesToRemove: Iterable[Any]) -> bool:
        """
        For each occurrence of an element `e` in `occurrencesToRemove`, removes one
        occurrence of `e` in `multisetToModify`.
        
        Equivalently, this method modifies `multisetToModify` so that `multisetToModify.count(e)` is set to `Math.max(0, multisetToModify.count(e) -
        Iterables.frequency(occurrencesToRemove, e))`.
        
        This is *not* the same as `multisetToModify.` Multiset.removeAll
        removeAll`(occurrencesToRemove)`, which removes all occurrences of elements that appear
        in `occurrencesToRemove`. However, this operation *is* equivalent to, albeit
        sometimes more efficient than, the following:
        
        ````for (E e : occurrencesToRemove) {
          multisetToModify.remove(e);`
        }```

        Returns
        - `True` if `multisetToModify` was changed as a result of this operation

        Since
        - 18.0 (present in 10.0 with a requirement that the second parameter be a `Multiset`)
        """
        ...


    @staticmethod
    def removeOccurrences(multisetToModify: "Multiset"[Any], occurrencesToRemove: "Multiset"[Any]) -> bool:
        """
        For each occurrence of an element `e` in `occurrencesToRemove`, removes one
        occurrence of `e` in `multisetToModify`.
        
        Equivalently, this method modifies `multisetToModify` so that `multisetToModify.count(e)` is set to `Math.max(0, multisetToModify.count(e) -
        occurrencesToRemove.count(e))`.
        
        This is *not* the same as `multisetToModify.` Multiset.removeAll
        removeAll`(occurrencesToRemove)`, which removes all occurrences of elements that appear
        in `occurrencesToRemove`. However, this operation *is* equivalent to, albeit
        sometimes more efficient than, the following:
        
        ````for (E e : occurrencesToRemove) {
          multisetToModify.remove(e);`
        }```

        Returns
        - `True` if `multisetToModify` was changed as a result of this operation

        Since
        - 10.0 (missing in 18.0 when only the overload taking an `Iterable` was present)
        """
        ...


    @staticmethod
    def copyHighestCountFirst(multiset: "Multiset"["E"]) -> "ImmutableMultiset"["E"]:
        """
        Returns a copy of `multiset` as an ImmutableMultiset whose iteration order puts
        the highest count first, with ties broken by the iteration order of the original multiset.

        Since
        - 11.0
        """
        ...



from .MapFont import MapFont
from .MapView import MapView
from .MapCursor import MapCursor
from .MapPalette import MapPalette
from .MapCanvas import MapCanvas
from .MapCursorCollection import MapCursorCollection
from .MinecraftFont import MinecraftFont
from .MapRenderer import MapRenderer


"""
Python module generated from Java source file org.bukkit.inventory.EnchantingInventory

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.inventory import *
from typing import Any, Callable, Iterable, Tuple


class EnchantingInventory(Inventory):
    """
    Interface to the inventory of an Enchantment Table.
    """

    def setItem(self, item: "ItemStack") -> None:
        """
        Set the item being enchanted.

        Arguments
        - item: The new item
        """
        ...


    def getItem(self) -> "ItemStack":
        """
        Get the item being enchanted.

        Returns
        - The current item.
        """
        ...


    def setSecondary(self, item: "ItemStack") -> None:
        """
        Set the secondary item being used for the enchant.

        Arguments
        - item: The new item
        """
        ...


    def getSecondary(self) -> "ItemStack":
        """
        Get the secondary item being used for the enchant.

        Returns
        - The second item
        """
        ...



"""
Python module generated from Java source file com.google.common.cache.CacheStats

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import MoreObjects
from com.google.common.base import Objects
from com.google.common.cache import *
from java.util.concurrent import Callable
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class CacheStats:
    """
    Statistics about the performance of a Cache. Instances of this class are immutable.
    
    Cache statistics are incremented according to the following rules:
    
    
      - When a cache lookup encounters an existing cache entry `hitCount` is incremented.
      - When a cache lookup first encounters a missing cache entry, a new entry is loaded.
          
            - After successfully loading an entry `missCount` and `loadSuccessCount`
                are incremented, and the total loading time, in nanoseconds, is added to `totalLoadTime`.
            - When an exception is thrown while loading an entry, `missCount` and `loadExceptionCount` are incremented, and the total loading time, in nanoseconds, is
                added to `totalLoadTime`.
            - Cache lookups that encounter a missing cache entry that is still loading will wait
                for loading to complete (whether successful or not) and then increment `missCount`.
          
      - When an entry is evicted from the cache, `evictionCount` is incremented.
      - No stats are modified when a cache entry is invalidated or manually removed.
      - No stats are modified by operations invoked on the Cache.asMap asMap view of
          the cache.
    
    
    A lookup is specifically defined as an invocation of one of the methods LoadingCache.get(Object), LoadingCache.getUnchecked(Object), Cache.get(Object,
    Callable), or LoadingCache.getAll(Iterable).

    Author(s)
    - Charles Fry

    Since
    - 10.0
    """

    def __init__(self, hitCount: int, missCount: int, loadSuccessCount: int, loadExceptionCount: int, totalLoadTime: int, evictionCount: int):
        """
        Constructs a new `CacheStats` instance.
        
        Five parameters of the same type in a row is a bad thing, but this class is not constructed
        by end users and is too fine-grained for a builder.
        """
        ...


    def requestCount(self) -> int:
        """
        Returns the number of times Cache lookup methods have returned either a cached or
        uncached value. This is defined as `hitCount + missCount`.
        
        **Note:** the values of the metrics are undefined in case of overflow (though it is
        guaranteed not to throw an exception). If you require specific handling, we recommend
        implementing your own stats collector.
        """
        ...


    def hitCount(self) -> int:
        """
        Returns the number of times Cache lookup methods have returned a cached value.
        """
        ...


    def hitRate(self) -> float:
        """
        Returns the ratio of cache requests which were hits. This is defined as `hitCount /
        requestCount`, or `1.0` when `requestCount == 0`. Note that `hitRate +
        missRate =~ 1.0`.
        """
        ...


    def missCount(self) -> int:
        """
        Returns the number of times Cache lookup methods have returned an uncached (newly
        loaded) value, or null. Multiple concurrent calls to Cache lookup methods on an absent
        value can result in multiple misses, all returning the results of a single cache load
        operation.
        """
        ...


    def missRate(self) -> float:
        """
        Returns the ratio of cache requests which were misses. This is defined as `missCount /
        requestCount`, or `0.0` when `requestCount == 0`. Note that `hitRate +
        missRate =~ 1.0`. Cache misses include all requests which weren't cache hits, including
        requests which resulted in either successful or failed loading attempts, and requests which
        waited for other threads to finish loading. It is thus the case that `missCount &gt;=
        loadSuccessCount + loadExceptionCount`. Multiple concurrent misses for the same key will result
        in a single load operation.
        """
        ...


    def loadCount(self) -> int:
        """
        Returns the total number of times that Cache lookup methods attempted to load new
        values. This includes both successful load operations and those that threw exceptions. This is
        defined as `loadSuccessCount + loadExceptionCount`.
        
        **Note:** the values of the metrics are undefined in case of overflow (though it is
        guaranteed not to throw an exception). If you require specific handling, we recommend
        implementing your own stats collector.
        """
        ...


    def loadSuccessCount(self) -> int:
        """
        Returns the number of times Cache lookup methods have successfully loaded a new value.
        This is usually incremented in conjunction with .missCount, though `missCount` is
        also incremented when an exception is encountered during cache loading (see .loadExceptionCount). Multiple concurrent misses for the same key will result in a single load
        operation. This may be incremented not in conjunction with `missCount` if the load occurs
        as a result of a refresh or if the cache loader returned more items than was requested. `missCount` may also be incremented not in conjunction with this (nor .loadExceptionCount) on calls to `getIfPresent`.
        """
        ...


    def loadExceptionCount(self) -> int:
        """
        Returns the number of times Cache lookup methods threw an exception while loading a new
        value. This is usually incremented in conjunction with `missCount`, though `missCount` is also incremented when cache loading completes successfully (see .loadSuccessCount). Multiple concurrent misses for the same key will result in a single load
        operation. This may be incremented not in conjunction with `missCount` if the load occurs
        as a result of a refresh or if the cache loader returned more items than was requested. `missCount` may also be incremented not in conjunction with this (nor .loadSuccessCount)
        on calls to `getIfPresent`.
        """
        ...


    def loadExceptionRate(self) -> float:
        """
        Returns the ratio of cache loading attempts which threw exceptions. This is defined as `loadExceptionCount / (loadSuccessCount + loadExceptionCount)`, or `0.0` when `loadSuccessCount + loadExceptionCount == 0`.
        
        **Note:** the values of the metrics are undefined in case of overflow (though it is
        guaranteed not to throw an exception). If you require specific handling, we recommend
        implementing your own stats collector.
        """
        ...


    def totalLoadTime(self) -> int:
        """
        Returns the total number of nanoseconds the cache has spent loading new values. This can be
        used to calculate the miss penalty. This value is increased every time `loadSuccessCount`
        or `loadExceptionCount` is incremented.
        """
        ...


    def averageLoadPenalty(self) -> float:
        """
        Returns the average time spent loading new values. This is defined as `totalLoadTime /
        (loadSuccessCount + loadExceptionCount)`.
        
        **Note:** the values of the metrics are undefined in case of overflow (though it is
        guaranteed not to throw an exception). If you require specific handling, we recommend
        implementing your own stats collector.
        """
        ...


    def evictionCount(self) -> int:
        """
        Returns the number of times an entry has been evicted. This count does not include manual
        Cache.invalidate invalidations.
        """
        ...


    def minus(self, other: "CacheStats") -> "CacheStats":
        """
        Returns a new `CacheStats` representing the difference between this `CacheStats`
        and `other`. Negative values, which aren't supported by `CacheStats` will be
        rounded up to zero.
        """
        ...


    def plus(self, other: "CacheStats") -> "CacheStats":
        """
        Returns a new `CacheStats` representing the sum of this `CacheStats` and `other`.
        
        **Note:** the values of the metrics are undefined in case of overflow (though it is
        guaranteed not to throw an exception). If you require specific handling, we recommend
        implementing your own stats collector.

        Since
        - 11.0
        """
        ...


    def hashCode(self) -> int:
        ...


    def equals(self, object: "Object") -> bool:
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file org.bukkit.SkullType

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit import *
from typing import Any, Callable, Iterable, Tuple


class SkullType(Enum):
    """
    Represents the different types of skulls.

    Deprecated
    - check Material instead
    """

    SKELETON = 0
    WITHER = 1
    ZOMBIE = 2
    PLAYER = 3
    CREEPER = 4
    DRAGON = 5
    PIGLIN = 6



"""
Python module generated from Java source file org.bukkit.util.EulerAngle

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.util import *
from typing import Any, Callable, Iterable, Tuple


class EulerAngle:
    """
    EulerAngle is used to represent 3 angles, one for each
    axis (x, y, z). The angles are in radians
    """

    ZERO = EulerAngle(0, 0, 0)
    """
    A EulerAngle with every axis set to 0
    """


    def __init__(self, x: float, y: float, z: float):
        """
        Creates a EularAngle with each axis set to the
        passed angle in radians

        Arguments
        - x: the angle for the x axis in radians
        - y: the angle for the y axis in radians
        - z: the angle for the z axis in radians
        """
        ...


    def getX(self) -> float:
        """
        Returns the angle on the x axis in radians

        Returns
        - the angle in radians
        """
        ...


    def getY(self) -> float:
        """
        Returns the angle on the y axis in radians

        Returns
        - the angle in radians
        """
        ...


    def getZ(self) -> float:
        """
        Returns the angle on the z axis in radians

        Returns
        - the angle in radians
        """
        ...


    def setX(self, x: float) -> "EulerAngle":
        """
        Return a EulerAngle which is the result of changing
        the x axis to the passed angle

        Arguments
        - x: the angle in radians

        Returns
        - the resultant EulerAngle
        """
        ...


    def setY(self, y: float) -> "EulerAngle":
        """
        Return a EulerAngle which is the result of changing
        the y axis to the passed angle

        Arguments
        - y: the angle in radians

        Returns
        - the resultant EulerAngle
        """
        ...


    def setZ(self, z: float) -> "EulerAngle":
        """
        Return a EulerAngle which is the result of changing
        the z axis to the passed angle

        Arguments
        - z: the angle in radians

        Returns
        - the resultant EulerAngle
        """
        ...


    def add(self, x: float, y: float, z: float) -> "EulerAngle":
        """
        Creates a new EulerAngle which is the result of adding
        the x, y, z components to this EulerAngle

        Arguments
        - x: the angle to add to the x axis in radians
        - y: the angle to add to the y axis in radians
        - z: the angle to add to the z axis in radians

        Returns
        - the resultant EulerAngle
        """
        ...


    def subtract(self, x: float, y: float, z: float) -> "EulerAngle":
        """
        Creates a new EulerAngle which is the result of subtracting
        the x, y, z components to this EulerAngle

        Arguments
        - x: the angle to subtract to the x axis in radians
        - y: the angle to subtract to the y axis in radians
        - z: the angle to subtract to the z axis in radians

        Returns
        - the resultant EulerAngle
        """
        ...


    def equals(self, o: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...



"""
Python module generated from Java source file org.bukkit.command.CommandSender

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import UUID
from org.bukkit import Server
from org.bukkit.command import *
from org.bukkit.permissions import Permissible
from typing import Any, Callable, Iterable, Tuple


class CommandSender(Permissible):

    def sendMessage(self, message: str) -> None:
        """
        Sends this sender a message

        Arguments
        - message: Message to be displayed
        """
        ...


    def sendMessage(self, *messages: Tuple[str, ...]) -> None:
        """
        Sends this sender multiple messages

        Arguments
        - messages: An array of messages to be displayed
        """
        ...


    def sendMessage(self, sender: "UUID", message: str) -> None:
        """
        Sends this sender a message

        Arguments
        - message: Message to be displayed
        - sender: The sender of this message
        """
        ...


    def sendMessage(self, sender: "UUID", *messages: Tuple[str, ...]) -> None:
        """
        Sends this sender multiple messages

        Arguments
        - messages: An array of messages to be displayed
        - sender: The sender of this message
        """
        ...


    def getServer(self) -> "Server":
        """
        Returns the server instance that this command is running on

        Returns
        - Server instance
        """
        ...


    def getName(self) -> str:
        """
        Gets the name of this command sender

        Returns
        - Name of the sender
        """
        ...


    def spigot(self) -> "Spigot":
        ...


    class Spigot:

        def sendMessage(self, component: "net.md_5.bungee.api.chat.BaseComponent") -> None:
            """
            Sends this sender a chat component.

            Arguments
            - component: the components to send
            """
            ...


        def sendMessage(self, *components: Tuple["net.md_5.bungee.api.chat.BaseComponent", ...]) -> None:
            """
            Sends an array of components as a single message to the sender.

            Arguments
            - components: the components to send
            """
            ...


        def sendMessage(self, sender: "UUID", component: "net.md_5.bungee.api.chat.BaseComponent") -> None:
            """
            Sends this sender a chat component.

            Arguments
            - component: the components to send
            - sender: the sender of the message
            """
            ...


        def sendMessage(self, sender: "UUID", *components: Tuple["net.md_5.bungee.api.chat.BaseComponent", ...]) -> None:
            """
            Sends an array of components as a single message to the sender.

            Arguments
            - components: the components to send
            - sender: the sender of the message
            """
            ...



"""
Python module generated from Java source file com.google.common.collect.GeneralRange

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import Objects
from com.google.common.collect import *
from com.google.errorprone.annotations.concurrent import LazyInit
from java.io import Serializable
from java.util import Comparator
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class GeneralRange(Serializable):
    """
    A generalized interval on any ordering, for internal use. Supports `null`. Unlike Range, this allows the use of an arbitrary comparator. This is designed for use in the
    implementation of subcollections of sorted collection types.
    
    Whenever possible, use `Range` instead, which is better supported.

    Author(s)
    - Louis Wasserman
    """

    def equals(self, obj: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file com.google.common.collect.ObjectArrays

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.lang.reflect import Array
from java.util import Arrays
from org.checkerframework.checker.nullness.qual import NonNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ObjectArrays:
    """
    Static utility methods pertaining to object arrays.

    Author(s)
    - Kevin Bourrillion

    Since
    - 2.0
    """

    @staticmethod
    def newArray(type: type["T"], length: int) -> list["T"]:
        """
        Returns a new array of the given length with the specified component type.

        Arguments
        - type: the component type
        - length: the length of the new array
        """
        ...


    @staticmethod
    def newArray(reference: list["T"], length: int) -> list["T"]:
        """
        Returns a new array of the given length with the same type as a reference array.

        Arguments
        - reference: any array of the desired type
        - length: the length of the new array
        """
        ...


    @staticmethod
    def concat(first: list["T"], second: list["T"], type: type["T"]) -> list["T"]:
        """
        Returns a new array that contains the concatenated contents of two arrays.

        Arguments
        - first: the first array of elements to concatenate
        - second: the second array of elements to concatenate
        - type: the component type of the returned array
        """
        ...


    @staticmethod
    def concat(element: "T", array: list["T"]) -> list["T"]:
        """
        Returns a new array that prepends `element` to `array`.

        Arguments
        - element: the element to prepend to the front of `array`
        - array: the array of elements to append

        Returns
        - an array whose size is one larger than `array`, with `element` occupying
            the first position, and the elements of `array` occupying the remaining elements.
        """
        ...


    @staticmethod
    def concat(array: list["T"], element: "T") -> list["T"]:
        """
        Returns a new array that appends `element` to `array`.

        Arguments
        - array: the array of elements to prepend
        - element: the element to append to the end

        Returns
        - an array whose size is one larger than `array`, with the same contents as `array`, plus `element` occupying the last position.
        """
        ...



"""
Python module generated from Java source file java.nio.file.SecureDirectoryStream

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import IOException
from java.nio.file import *
from java.nio.file.attribute import *
from typing import Any, Callable, Iterable, Tuple


class SecureDirectoryStream(DirectoryStream):

    def newDirectoryStream(self, path: "T", *options: Tuple["LinkOption", ...]) -> "SecureDirectoryStream"["T"]:
        """
        Opens the directory identified by the given path, returning a `SecureDirectoryStream` to iterate over the entries in the directory.
        
         This method works in exactly the manner specified by the Files.newDirectoryStream(Path) newDirectoryStream method for the case that
        the `path` parameter is an Path.isAbsolute absolute path.
        When the parameter is a relative path then the directory to open is
        relative to this open directory. The LinkOption.NOFOLLOW_LINKS NOFOLLOW_LINKS option may be used to
        ensure that this method fails if the file is a symbolic link.
        
         The new directory stream, once created, is not dependent upon the
        directory stream used to create it. Closing this directory stream has no
        effect upon newly created directory stream.

        Arguments
        - path: the path to the directory to open
        - options: options indicating how symbolic links are handled

        Returns
        - a new and open `SecureDirectoryStream` object

        Raises
        - ClosedDirectoryStreamException: if the directory stream is closed
        - NotDirectoryException: if the file could not otherwise be opened because it is not
                 a directory *(optional specific exception)*
        - IOException: if an I/O error occurs
        - SecurityException: In the case of the default provider, and a security manager is
                 installed, the SecurityManager.checkRead(String) checkRead
                 method is invoked to check read access to the directory.
        """
        ...


    def newByteChannel(self, path: "T", options: set["OpenOption"], *attrs: Tuple["FileAttribute"[Any], ...]) -> "SeekableByteChannel":
        """
        Opens or creates a file in this directory, returning a seekable byte
        channel to access the file.
        
         This method works in exactly the manner specified by the Files.newByteChannel Files.newByteChannel method for the
        case that the `path` parameter is an Path.isAbsolute absolute
        path. When the parameter is a relative path then the file to open or
        create is relative to this open directory. In addition to the options
        defined by the `Files.newByteChannel` method, the LinkOption.NOFOLLOW_LINKS NOFOLLOW_LINKS option may be used to
        ensure that this method fails if the file is a symbolic link.
        
         The channel, once created, is not dependent upon the directory stream
        used to create it. Closing this directory stream has no effect upon the
        channel.

        Arguments
        - path: the path of the file to open or create
        - options: options specifying how the file is opened
        - attrs: an optional list of attributes to set atomically when creating
                 the file

        Returns
        - the seekable byte channel

        Raises
        - ClosedDirectoryStreamException: if the directory stream is closed
        - IllegalArgumentException: if the set contains an invalid combination of options
        - UnsupportedOperationException: if an unsupported open option is specified or the array contains
                 attributes that cannot be set atomically when creating the file
        - FileAlreadyExistsException: if a file of that name already exists and the StandardOpenOption.CREATE_NEW CREATE_NEW option is specified
                 *(optional specific exception)*
        - IOException: if an I/O error occurs
        - SecurityException: In the case of the default provider, and a security manager is
                 installed, the SecurityManager.checkRead(String) checkRead
                 method is invoked to check read access to the path if the file
                 is opened for reading. The SecurityManager.checkWrite(String)
                 checkWrite method is invoked to check write access to the path
                 if the file is opened for writing.
        """
        ...


    def deleteFile(self, path: "T") -> None:
        """
        Deletes a file.
        
         Unlike the Files.delete delete() method, this method does
        not first examine the file to determine if the file is a directory.
        Whether a directory is deleted by this method is system dependent and
        therefore not specified. If the file is a symbolic link, then the link
        itself, not the final target of the link, is deleted. When the
        parameter is a relative path then the file to delete is relative to
        this open directory.

        Arguments
        - path: the path of the file to delete

        Raises
        - ClosedDirectoryStreamException: if the directory stream is closed
        - NoSuchFileException: if the file does not exist *(optional specific exception)*
        - IOException: if an I/O error occurs
        - SecurityException: In the case of the default provider, and a security manager is
                 installed, the SecurityManager.checkDelete(String) checkDelete
                 method is invoked to check delete access to the file
        """
        ...


    def deleteDirectory(self, path: "T") -> None:
        """
        Deletes a directory.
        
         Unlike the Files.delete delete() method, this method
        does not first examine the file to determine if the file is a directory.
        Whether non-directories are deleted by this method is system dependent and
        therefore not specified. When the parameter is a relative path then the
        directory to delete is relative to this open directory.

        Arguments
        - path: the path of the directory to delete

        Raises
        - ClosedDirectoryStreamException: if the directory stream is closed
        - NoSuchFileException: if the directory does not exist *(optional specific exception)*
        - DirectoryNotEmptyException: if the directory could not otherwise be deleted because it is
                 not empty *(optional specific exception)*
        - IOException: if an I/O error occurs
        - SecurityException: In the case of the default provider, and a security manager is
                 installed, the SecurityManager.checkDelete(String) checkDelete
                 method is invoked to check delete access to the directory
        """
        ...


    def move(self, srcpath: "T", targetdir: "SecureDirectoryStream"["T"], targetpath: "T") -> None:
        """
        Move a file from this directory to another directory.
        
         This method works in a similar manner to Files.move move
        method when the StandardCopyOption.ATOMIC_MOVE ATOMIC_MOVE option
        is specified. That is, this method moves a file as an atomic file system
        operation. If the `srcpath` parameter is an Path.isAbsolute
        absolute path then it locates the source file. If the parameter is a
        relative path then it is located relative to this open directory. If
        the `targetpath` parameter is absolute then it locates the target
        file (the `targetdir` parameter is ignored). If the parameter is
        a relative path it is located relative to the open directory identified
        by the `targetdir` parameter. In all cases, if the target file
        exists then it is implementation specific if it is replaced or this
        method fails.

        Arguments
        - srcpath: the name of the file to move
        - targetdir: the destination directory
        - targetpath: the name to give the file in the destination directory

        Raises
        - ClosedDirectoryStreamException: if this or the target directory stream is closed
        - FileAlreadyExistsException: if the file already exists in the target directory and cannot
                 be replaced *(optional specific exception)*
        - AtomicMoveNotSupportedException: if the file cannot be moved as an atomic file system operation
        - IOException: if an I/O error occurs
        - SecurityException: In the case of the default provider, and a security manager is
                 installed, the SecurityManager.checkWrite(String) checkWrite
                 method is invoked to check write access to both the source and
                 target file.
        """
        ...


    def getFileAttributeView(self, type: type["V"]) -> "V":
        """
        Returns a new file attribute view to access the file attributes of this
        directory.
        
         The resulting file attribute view can be used to read or update the
        attributes of this (open) directory. The `type` parameter specifies
        the type of the attribute view and the method returns an instance of that
        type if supported. Invoking this method to obtain a BasicFileAttributeView always returns an instance of that class that is
        bound to this open directory.
        
         The state of resulting file attribute view is intimately connected
        to this directory stream. Once the directory stream is .close closed,
        then all methods to read or update attributes will throw ClosedDirectoryStreamException ClosedDirectoryStreamException.
        
        Type `<V>`: The `FileAttributeView` type

        Arguments
        - type: the `Class` object corresponding to the file attribute view

        Returns
        - a new file attribute view of the specified type bound to
                 this directory stream, or `null` if the attribute view
                 type is not available
        """
        ...


    def getFileAttributeView(self, path: "T", type: type["V"], *options: Tuple["LinkOption", ...]) -> "V":
        """
        Returns a new file attribute view to access the file attributes of a file
        in this directory.
        
         The resulting file attribute view can be used to read or update the
        attributes of file in this directory. The `type` parameter specifies
        the type of the attribute view and the method returns an instance of that
        type if supported. Invoking this method to obtain a BasicFileAttributeView always returns an instance of that class that is
        bound to the file in the directory.
        
         The state of resulting file attribute view is intimately connected
        to this directory stream. Once the directory stream .close closed,
        then all methods to read or update attributes will throw ClosedDirectoryStreamException ClosedDirectoryStreamException. The
        file is not required to exist at the time that the file attribute view
        is created but methods to read or update attributes of the file will
        fail when invoked and the file does not exist.
        
        Type `<V>`: The `FileAttributeView` type

        Arguments
        - path: the path of the file
        - type: the `Class` object corresponding to the file attribute view
        - options: options indicating how symbolic links are handled

        Returns
        - a new file attribute view of the specified type bound to a
                 this directory stream, or `null` if the attribute view
                 type is not available
        """
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.serializer.NumberAnchorGenerator

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.text import NumberFormat
from org.yaml.snakeyaml.nodes import Node
from org.yaml.snakeyaml.serializer import *
from typing import Any, Callable, Iterable, Tuple


class NumberAnchorGenerator(AnchorGenerator):

    def __init__(self, lastAnchorId: int):
        ...


    def nextAnchor(self, node: "Node") -> str:
        ...



"""
Python module generated from Java source file org.bukkit.plugin.SimplePluginManager

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from com.google.common.collect import ImmutableSet
from com.google.common.graph import GraphBuilder
from com.google.common.graph import Graphs
from com.google.common.graph import MutableGraph
from java.io import File
from java.lang.reflect import Constructor
from java.lang.reflect import Method
from java.lang.reflect import Modifier
from java.util import Iterator
from java.util import LinkedHashSet
from java.util import Locale
from java.util import WeakHashMap
from java.util.regex import Matcher
from java.util.regex import Pattern
from org.bukkit import Server
from org.bukkit import World
from org.bukkit.command import Command
from org.bukkit.command import PluginCommandYamlParser
from org.bukkit.command import SimpleCommandMap
from org.bukkit.event import Event
from org.bukkit.event import EventPriority
from org.bukkit.event import HandlerList
from org.bukkit.event import Listener
from org.bukkit.permissions import Permissible
from org.bukkit.permissions import Permission
from org.bukkit.permissions import PermissionDefault
from org.bukkit.plugin import *
from org.bukkit.util import FileUtil
from typing import Any, Callable, Iterable, Tuple


class SimplePluginManager(PluginManager):
    """
    Handles all plugin management from the Server
    """

    def __init__(self, instance: "Server", commandMap: "SimpleCommandMap"):
        ...


    def registerInterface(self, loader: type["PluginLoader"]) -> None:
        """
        Registers the specified plugin loader

        Arguments
        - loader: Class name of the PluginLoader to register

        Raises
        - IllegalArgumentException: Thrown when the given Class is not a
            valid PluginLoader
        """
        ...


    def loadPlugins(self, directory: "File") -> list["Plugin"]:
        """
        Loads the plugins contained within the specified directory

        Arguments
        - directory: Directory to check for plugins

        Returns
        - A list of all plugins loaded
        """
        ...


    def loadPlugins(self, files: list["File"]) -> list["Plugin"]:
        """
        Loads the plugins in the list of the files

        Arguments
        - files: List of files containing plugins to load

        Returns
        - A list of all plugins loaded
        """
        ...


    def loadPlugin(self, file: "File") -> "Plugin":
        """
        Loads the plugin in the specified file
        
        File must be valid according to the current enabled Plugin interfaces

        Arguments
        - file: File containing the plugin to load

        Returns
        - The Plugin loaded, or null if it was invalid

        Raises
        - InvalidPluginException: Thrown when the specified file is not a
            valid plugin
        - UnknownDependencyException: If a required dependency could not
            be found
        """
        ...


    def getPlugin(self, name: str) -> "Plugin":
        """
        Checks if the given plugin is loaded and returns it when applicable
        
        Please note that the name of the plugin is case-sensitive

        Arguments
        - name: Name of the plugin to check

        Returns
        - Plugin if it exists, otherwise null
        """
        ...


    def getPlugins(self) -> list["Plugin"]:
        ...


    def isPluginEnabled(self, name: str) -> bool:
        """
        Checks if the given plugin is enabled or not
        
        Please note that the name of the plugin is case-sensitive.

        Arguments
        - name: Name of the plugin to check

        Returns
        - True if the plugin is enabled, otherwise False
        """
        ...


    def isPluginEnabled(self, plugin: "Plugin") -> bool:
        """
        Checks if the given plugin is enabled or not

        Arguments
        - plugin: Plugin to check

        Returns
        - True if the plugin is enabled, otherwise False
        """
        ...


    def enablePlugin(self, plugin: "Plugin") -> None:
        ...


    def disablePlugins(self) -> None:
        ...


    def disablePlugin(self, plugin: "Plugin") -> None:
        ...


    def clearPlugins(self) -> None:
        ...


    def callEvent(self, event: "Event") -> None:
        """
        Calls an event with the given details.

        Arguments
        - event: Event details
        """
        ...


    def registerEvents(self, listener: "Listener", plugin: "Plugin") -> None:
        ...


    def registerEvent(self, event: type["Event"], listener: "Listener", priority: "EventPriority", executor: "EventExecutor", plugin: "Plugin") -> None:
        ...


    def registerEvent(self, event: type["Event"], listener: "Listener", priority: "EventPriority", executor: "EventExecutor", plugin: "Plugin", ignoreCancelled: bool) -> None:
        """
        Registers the given event to the specified listener using a directly
        passed EventExecutor

        Arguments
        - event: Event class to register
        - listener: PlayerListener to register
        - priority: Priority of this event
        - executor: EventExecutor to register
        - plugin: Plugin to register
        - ignoreCancelled: Do not call executor if event was already
            cancelled
        """
        ...


    def getPermission(self, name: str) -> "Permission":
        ...


    def addPermission(self, perm: "Permission") -> None:
        ...


    def addPermission(self, perm: "Permission", dirty: bool) -> None:
        ...


    def getDefaultPermissions(self, op: bool) -> set["Permission"]:
        ...


    def removePermission(self, perm: "Permission") -> None:
        ...


    def removePermission(self, name: str) -> None:
        ...


    def recalculatePermissionDefaults(self, perm: "Permission") -> None:
        ...


    def dirtyPermissibles(self) -> None:
        ...


    def subscribeToPermission(self, permission: str, permissible: "Permissible") -> None:
        ...


    def unsubscribeFromPermission(self, permission: str, permissible: "Permissible") -> None:
        ...


    def getPermissionSubscriptions(self, permission: str) -> set["Permissible"]:
        ...


    def subscribeToDefaultPerms(self, op: bool, permissible: "Permissible") -> None:
        ...


    def unsubscribeFromDefaultPerms(self, op: bool, permissible: "Permissible") -> None:
        ...


    def getDefaultPermSubscriptions(self, op: bool) -> set["Permissible"]:
        ...


    def getPermissions(self) -> set["Permission"]:
        ...


    def isTransitiveDepend(self, plugin: "PluginDescriptionFile", depend: "PluginDescriptionFile") -> bool:
        ...


    def useTimings(self) -> bool:
        ...


    def useTimings(self, use: bool) -> None:
        """
        Sets whether or not per event timing code should be used

        Arguments
        - use: True if per event timing code should be used
        """
        ...



"""
Python module generated from Java source file org.bukkit.util.RayTraceResult

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from java.util import Objects
from org.bukkit.block import Block
from org.bukkit.block import BlockFace
from org.bukkit.entity import Entity
from org.bukkit.util import *
from typing import Any, Callable, Iterable, Tuple


class RayTraceResult:
    """
    The hit result of a ray trace.
    
    Only the hit position is guaranteed to always be available. The availability
    of the other attributes depends on what got hit and on the context in which
    the ray trace was performed.
    """

    def __init__(self, hitPosition: "Vector"):
        """
        Creates a RayTraceResult.

        Arguments
        - hitPosition: the hit position
        """
        ...


    def __init__(self, hitPosition: "Vector", hitBlockFace: "BlockFace"):
        """
        Creates a RayTraceResult.

        Arguments
        - hitPosition: the hit position
        - hitBlockFace: the hit block face
        """
        ...


    def __init__(self, hitPosition: "Vector", hitBlock: "Block", hitBlockFace: "BlockFace"):
        """
        Creates a RayTraceResult.

        Arguments
        - hitPosition: the hit position
        - hitBlock: the hit block
        - hitBlockFace: the hit block face
        """
        ...


    def __init__(self, hitPosition: "Vector", hitEntity: "Entity"):
        """
        Creates a RayTraceResult.

        Arguments
        - hitPosition: the hit position
        - hitEntity: the hit entity
        """
        ...


    def __init__(self, hitPosition: "Vector", hitEntity: "Entity", hitBlockFace: "BlockFace"):
        """
        Creates a RayTraceResult.

        Arguments
        - hitPosition: the hit position
        - hitEntity: the hit entity
        - hitBlockFace: the hit block face
        """
        ...


    def getHitPosition(self) -> "Vector":
        """
        Gets the exact position of the hit.

        Returns
        - a copy of the exact hit position
        """
        ...


    def getHitBlock(self) -> "Block":
        """
        Gets the hit block.

        Returns
        - the hit block, or `null` if not available
        """
        ...


    def getHitBlockFace(self) -> "BlockFace":
        """
        Gets the hit block face.

        Returns
        - the hit block face, or `null` if not available
        """
        ...


    def getHitEntity(self) -> "Entity":
        """
        Gets the hit entity.

        Returns
        - the hit entity, or `null` if not available
        """
        ...


    def hashCode(self) -> int:
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file org.bukkit.command.defaults.PluginsCommand

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Arrays
from java.util import Collections
from org.bukkit import Bukkit
from org.bukkit import ChatColor
from org.bukkit.command import CommandSender
from org.bukkit.command.defaults import *
from org.bukkit.plugin import Plugin
from typing import Any, Callable, Iterable, Tuple


class PluginsCommand(BukkitCommand):

    def __init__(self, name: str):
        ...


    def execute(self, sender: "CommandSender", currentAlias: str, args: list[str]) -> bool:
        ...


    def tabComplete(self, sender: "CommandSender", alias: str, args: list[str]) -> list[str]:
        ...



"""
Python module generated from Java source file com.google.common.collect.TreeRangeSet

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.base import MoreObjects
from com.google.common.collect import *
from com.google.errorprone.annotations.concurrent import LazyInit
from java.io import Serializable
from java.util import Comparator
from java.util import Iterator
from java.util import NavigableMap
from java.util import NoSuchElementException
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class TreeRangeSet(AbstractRangeSet, Serializable):
    """
    An implementation of RangeSet backed by a TreeMap.

    Author(s)
    - Louis Wasserman

    Since
    - 14.0
    """

    @staticmethod
    def create() -> "TreeRangeSet"["C"]:
        """
        Creates an empty `TreeRangeSet` instance.
        """
        ...


    @staticmethod
    def create(rangeSet: "RangeSet"["C"]) -> "TreeRangeSet"["C"]:
        """
        Returns a `TreeRangeSet` initialized with the ranges in the specified range set.
        """
        ...


    @staticmethod
    def create(ranges: Iterable["Range"["C"]]) -> "TreeRangeSet"["C"]:
        """
        Returns a `TreeRangeSet` representing the union of the specified ranges.
        
        This is the smallest `RangeSet` which encloses each of the specified ranges. An
        element will be contained in this `RangeSet` if and only if it is contained in at least
        one `Range` in `ranges`.

        Since
        - 21.0
        """
        ...


    def asRanges(self) -> set["Range"["C"]]:
        ...


    def asDescendingSetOfRanges(self) -> set["Range"["C"]]:
        ...


    def rangeContaining(self, value: "C") -> "Range"["C"]:
        ...


    def intersects(self, range: "Range"["C"]) -> bool:
        ...


    def encloses(self, range: "Range"["C"]) -> bool:
        ...


    def span(self) -> "Range"["C"]:
        ...


    def add(self, rangeToAdd: "Range"["C"]) -> None:
        ...


    def remove(self, rangeToRemove: "Range"["C"]) -> None:
        ...


    def complement(self) -> "RangeSet"["C"]:
        ...


    def subRangeSet(self, view: "Range"["C"]) -> "RangeSet"["C"]:
        ...
